@@ -19,6 +19,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "_gocr.h"
 #include "gocr.h"
+#include <string.h>
 
 gocr_Box *currentbox;
 
@@ -60,7 +61,7 @@ int gocr_charBegin ( void ) {
 	began = NEWCHAR;
 
 	/* we are working in the block */
-	workimage = &currentblock->image;
+	workimage = currentblock->image;
 
 	currentbox = (gocr_Box *) malloc(sizeof(gocr_Box));
 	if (!currentbox) {
@@ -124,7 +125,7 @@ memory use to 1/8, and yet keep all information */
 		return -1;
 	}
 	/* we have to create a 2nd image to allow the pixel by pixel selection */
-	if (_gocr_imageSharedCopy(&currentblock->image, currentbox->x0, 
+	if (_gocr_imageSharedCopy(currentblock->image, currentbox->x0, 
 			currentbox->y0, currentbox->x1, currentbox->y1, 
 			currentbox->image) != 0) {
 		_gocr_printf(1, FUNCTION, "Could not copy image\n");
@@ -164,6 +165,8 @@ information. */
 		gocr_runModule(charRecognizer, box);
 		gocr_runModule(contextCorrection, box);
 		gocr_imageFree(box->image);
+		free(box->image);
+		box->image = NULL;
 #endif
 	}
 	list_app(&currentblock->boxlist, currentbox);
@@ -575,11 +578,13 @@ int gocr_charSetPixel ( int action, int x, int y ) {
 	else {			/* unselecting, we may have to shrink */
 		int i;
 		if (x == currentbox->x0) {
-			for (; currentbox->x0 <= currentbox->x1; currentbox->x0++)
+			for (; currentbox->x0 <= currentbox->x1; currentbox->x0++) {
 				for (i = currentbox->y0; i <= currentbox->y1; i++)
 					if (_gocr_private1(workimage,
 					    currentbox->x0, i) == 1)
-						goto x0end;	/* gotos are easier here */
+						goto x0end;	/* gotos are easier 
+						here */
+			}
 		      x0end:
 		}
 		else if (x == currentbox->x1) {
