@@ -24,8 +24,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "_gocr.h"
 #include "gocr.h"
 
-List liblist;
-List modulefunclist[allModules];
+gocr_List liblist;
+gocr_List modulefunclist[allModules];
 gocr_Block empty;
 
 static gocr_ModuleId 		currentlibid = 0;
@@ -50,11 +50,14 @@ struct _gocr_modulefunction {
 	void 			*data;
 
 	union {
-		void (*imageFilter) (gocr_Image * img, void *v);
-		void (*blockFinder) (gocr_Image * img, void *v);
-		int (*charFinder) (gocr_Block * b, void *v);
-		int (*charRecognizer) (gocr_Box * b, void *v);
-		void (*contextCorrection) (gocr_Box * b, void *v);
+		void (*imageFilter) (gocr_Image *src, void *v);		
+//		void (*imageFilter) (gocr_Image *src, gocr_Image *target, void *v);
+		void (*blockFinder) (gocr_Image *img, void *v);
+//		int (*blockFinder) (gocr_Image *img, gocr_Block **b, void *v);
+		int (*charFinder) (gocr_Block *b, void *v);
+//		int (*charFinder) (gocr_Block *bl, gocr_Box **b, void *v);
+		int (*charRecognizer) (gocr_Box *b, void *v);
+		void (*contextCorrection) (gocr_Box *b, void *v);
 		void (*outputFormatter) (gocr_Block **bl, void *v);
 	} func;
 };
@@ -148,7 +151,7 @@ gocr_ModuleId gocr_moduleLoad ( char *filename ) {
 			     filename, lt_dlerror());
 		return -1;
 	}
-
+	
 	/* try to open an init function */
 	init_func = lt_dlsym(l->handle, "initModule");
 	if (init_func) {
@@ -588,13 +591,16 @@ int gocr_runModuleFunction ( gocr_ModuleFunctionId id, ... ) {
 	/* run it */
 	switch (t) {
 	  case imageFilter:{
-		gocr_Image *image;
+		gocr_Image *src;
+		gocr_Image *target;
 		void *v;
 
 		va_start(argp, id);
-		image = va_arg(argp, gocr_Image *);
+		src = va_arg(argp, gocr_Image *);
+		target = va_arg(argp, gocr_Image *);
 		v = va_arg(argp, void *);
-		m->func.imageFilter(image, v);
+//		m->func.imageFilter(src, target, v);
+//		m->func.imageFilter(src, v);
 		va_end(argp);
 		}
 		break;
@@ -818,24 +824,6 @@ int gocr_runAllModules ( void ) {
 	/* and free memory */
 	free(blist);
 
-/*** perhaps this is not the correct place to put it (imageWrite) */
-	for_each_data(&blocklist) {
-		block = (gocr_Block *)list_get_current(&blocklist);
-		for_each_data(&block->boxlist) {
-			box = (gocr_Box *)list_get_current(&block->boxlist);
-/* BUG: doesn't work with shared images and segfaults.
-			gocr_imageFree(box->image); */
-			for_each_data(&box->possible) {
-				gocr_Char *c = (gocr_Char *)list_get_current(&box->possible);
-				free(c);
-			} end_for_each(&box->possible);
-			list_free(&box->possible);
-			free(box);
-		} end_for_each(&block->boxlist);
-		list_free(&block->boxlist);
-		free(block);
-	} end_for_each(&blocklist);
-	list_free(&blocklist);
 	_data.lock = 0;
 	return 0;
 }
