@@ -7,6 +7,10 @@
 #include "../include/gocr.h"
 #include "../include/gocr_module.h"
 #include "../include/gocr_char.h"
+#include <ltdl.h>
+#include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 
 /* this is hack, disregard */
 void none( void ) {
@@ -16,6 +20,9 @@ void none( void ) {
 
 int main (int argc, char **argv) {
 	gocr_ModuleId		m;
+	char *x;
+
+ LTDL_SET_PRELOADED_SYMBOLS();
 
 	if (gocr_init(argc, argv) != 0) {
 		printf("Could not init gocr\n");
@@ -25,7 +32,7 @@ int main (int argc, char **argv) {
 	gocr_setAttribute(VERBOSE, (void *)3);
 	
 	/* modules */
-	if ( (m = gocr_moduleLoad("/home/gocr/modules/modules/libtester.so")) == -1 ) {
+	if ( (m = gocr_moduleLoad("/home/gocr/modules/modules/libtester.la")) == -1 ) {
 		fprintf(stderr, "You should get the mdk tester at gocr's homepage.\n"
 				"If you already have it, edit module.c and make sure the"
 				"path is correct.\n");
@@ -40,10 +47,9 @@ int main (int argc, char **argv) {
 
 	gocr_imageLoad("small.pbm", (void *)GOCR_BW);
 	gocr_runAllModules();
-//	gocr_charBegin();	
-//	gocr_charEnd();
 
 	gocr_mainImageWriteWithData("module.ppm");
+	gocr_imageClose();
 	gocr_finalize();
 	return 0;
 }
