@@ -74,7 +74,7 @@ void ViewFile(GtkWidget *wid, gpointer Data)
   if (nSelectedRow==-1)     return;
 
   gtk_clist_get_text(list_file,nSelectedRow,0,&Name);
-  sprintf(cmd,"%s %s &",Viewer,Name);
+  sprintf(cmd,"%s \"%s\" &",Viewer,Name);
 
   system (cmd);
 }
