@@ -11,7 +11,7 @@
     Tell me, if you have an Amiga and you can give answer
     to the following questions.
             
-                 jschulen@gmx.de
+                 Joerg Schulenburg, see README for EMAIL-address
      
  */
  
