@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2006 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,15 +29,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define Blen  256
 
-// load database into boxlist
-// this is added in version v0.2.4 
-//   as alternate engine comparing chars with database
+// load boxes from database into boxlist (for faster access)
+//   used as alternate engine, comparing chars with database
 int load_db(void) {
   FILE *f1;
   char s1[Blen+1],
-       s2[Blen+1] = "./db/", /* replace by constant! defined by configure */
+       s2[Blen+1] = "./db/", /* ToDo: replace by constant! by configure */
        *s3;
-  int i, j, ii, i2;
+  int i, j, ii, i2, line;
   struct box *box1;
   pix *pp;
 
@@ -53,25 +52,29 @@ int load_db(void) {
     return 1;
   }
 
+  line = 0; /* line counter for better error report */
   for (ii = 0; !feof(f1); ii++) {
 /* bbg: should write a better input routine */
-    if (!fgets(s1, Blen, f1)) break;
+    if (!fgets(s1, Blen, f1)) break; line++;
     j = strlen(s1);
-    if (j == 0)
-      continue;
+    /* remove carriage return sequences from line */
+    while (j > 0 && (s1[j - 1] == '\r' || s1[j - 1] == '\n'))
+      s1[--j] = 0;
+    if (!j)
+      continue;   /* skip empty line */
     /* copy file name */
     for (i = 0; i < j && i+i2 < Blen && strchr(" \t,;",s1[i]) == 0; i++)
       s2[i2 + i] = s1[i];
     s2[i2+i]=0;
     /* skip spaces */
     for (; i < j && strchr(" \t",s1[i]) != 0; i++);
-/* by now: read pix, fill box, goto next ??? */
+    /* by now: read pix, fill box, goto next ??? */
     pp = (pix *)malloc(sizeof(pix));
     if( !pp ) fprintf(stderr,"malloc error in load_db pix\n");
 
     readpgm(s2, pp, 0 * JOB->cfg.verbose);
 
-    box1 = (struct box *)malloc(sizeof(struct box));
+    box1 = (struct box *)malloc_box(NULL);
     if(!box1) fprintf(stderr,"malloc error in load_db box1\n");
     box1->x0 = 0;
     box1->x1 = pp->x-1;       // white border 1 pixel width
@@ -81,34 +84,39 @@ int load_db(void) {
     box1->y = 1;
     box1->dots = 0;
     box1->c = 0;
-    box1->obj = NULL;
-    if (s1[i]=='"'){
-      j=strchr(s1+i+1,'"')-(s1+i+1);
-      if (j>=1 && j<40) {
+    box1->modifier = 0; /* ToDo: obsolete */
+    box1->tas[0]=NULL;
+    box1->tac[0]=0;
+    box1->wac[0]=100; /* really 100% sure? */
+    box1->num_ac=1;
+    if (s1[i]=='"'){  /* parse a string */
+      j=strrchr(s1+i+1,'"')-(s1+i+1); /* we only look for first and last "" */
+      if (j>=1) {
         s3=(char *)malloc(j+1);
+        if (!s3) fprintf (stderr, "malloc error in load_db s3\n");
         if (s3) {
           memcpy(s3,s1+i+1,j);
           s3[j]=0;
-          box1->obj=s3;
+          box1->tas[0]=s3;
           // fprintf(stderr,"\nstring=%s",s3);
         }
-      }
+      } else { fprintf(stderr,"load_db: string parse error L%d\n",line); }
     } else {
-      box1->c = s1[i];      /* try to interpret as ASCII */
+      box1->tac[0] = box1->c = s1[i];      /* try to interpret as ASCII */
+      /* we can live without hexcode in future if we use UTF8-strings */
       s3=s1+i;
       j=strtol( s1+i, &s3, 16); /* try to read 4 to 8 digit hex unicode */
-      if( j && i+3<=Blen && s3-s1-i>3 ) box1->c = j;
+      /* if its an hexcode, ASCII interpretation is overwritten */
+      if( j && i+3<=Blen && s3-s1-i>3 ) box1->tac[0] = box1->c = j;
       // fprintf(stderr,"\nhexcode=%04x=%04x %d",(int)j,(int)box1->c,s3-s1-i);
     }
-    box1->ac = '_';
     box1->num = 0;
     box1->line = -1;
-    box1->m1 = 0;  /* should be given too in the database! change it */
+    box1->m1 = 0;  /* ToDo: should be given too in the database! */
     box1->m2 = 0;
     box1->m3 = 0;
     box1->m4 = 0;
     box1->p = pp;
-    box1->num_ac=0;
     list_app(&JOB->tmp.dblist, box1);	// append to list
 #if 0
      out_x(box1);
@@ -132,22 +140,30 @@ int store_db(struct box *box1) {
   i2=strlen(s2);
   if (JOB->cfg.verbose)
     fprintf(stderr, "# expand database %s ... ",s2);
-
-  sprintf(s3,"db_%lu.pbm",(unsigned long)time(NULL));
+ 
+  /* name generation can cause problems, if called twice within a second */
+  if (box1->num_ac && box1->tas[0])
+    sprintf(s3,"db_%04x_%lu.pbm", (unsigned int)box1->tas[0][0],
+                                  (unsigned long)time(NULL));
+  else
+    sprintf(s3,"db_%04x_%lu.pbm", (unsigned int)box1->c,
+                                  (unsigned long)time(NULL));
+  /* ToDo: the file name may be not unique */
   strncpy(s2+i2,"db.lst",Blen-i2);s2[Blen]=0;
   f1 = fopen(s2, "a");
   if (!f1) {
     fprintf(stderr, " could not access %s\n",s2);
     return 1;
   }
-  strncpy(s2+i2,s3,strlen(s3));s2[i2+strlen(s3)]=0;
+  strncpy(s2+i2,s3,strlen(s3)); s2[i2+strlen(s3)]=0;
   /* store image and infos about the char */
+  /* ToDo: store the vector list instead of the pixelarray */
   
   if (JOB->cfg.verbose)
     fprintf(stderr, "add file %s to database",s3);
   dx=box1->x1-box1->x0+1;
   dy=box1->y1-box1->y0+1;
-  b.p = malloc( dx * dy * sizeof(unsigned char) );
+  b.p = (unsigned char *) malloc( dx * dy );
   if( !b.p ){
     fprintf( stderr, "\nFATAL: malloc failed, skip store_db" );
     return 2;
@@ -155,12 +171,13 @@ int store_db(struct box *box1) {
   if (copybox(box1->p, box1->x0, box1->y0, dx, dy, &b, dx * dy))
     return -1;
                           
-  writepbm(s2,&b);
+  writepbm(s2,&b); /* What is to do on error? */
   free(b.p);
   
   // some infos about box1->m1,m2,m3,m4 should added (base line, high etc.)
-  if (box1->obj) {
-    fprintf(f1, "%s \"%s\"\n",s3,box1->obj);
+  if (box1->num_ac && box1->tas[0]) {
+    fprintf(f1, "%s \"%s\"\n",s3,box1->tas[0]);
+    /* ToDo: what if tas contains '"'? */
   } else {
     if( (box1->c >= '0' && box1->c <= '9')
      || (box1->c >= 'A' && box1->c <= 'Z')
@@ -181,6 +198,74 @@ int store_db(struct box *box1) {
   return 0;
 }
 
+/* function is only for user prompt on console to identify chars
+   it prints out a part of pixmap b at point x0,y0 to stderr
+   using dots .,; if no pixel, and @xoO for pixels
+ */
+void out_env(struct box *px ){
+  int x0,y0,x1,y1,dx,dy,x,y,x2,y2,yy0,tx,ty,i,cs;
+  char c1, c2; pix *b;
+  cs=JOB->cfg.cs;
+  yy0=px->y0;
+  { /* overwrite rest of arguments */
+    b=px->p;
+    x0=px->x0; x1=px->x1; dx=x1-x0+1;
+    y0=px->y0; y1=px->y1; dy=y1-y0+1;
+    y0-=2; y1+=2; 
+    if (px->m4 && y0>px->m1) y0=px->m1;
+    if (px->m4 && y1<px->m4) y1=px->m4;
+    if (x1-x0+1<52) { x0-=10; x1+=10; } /* fragment? expand frame */
+    if (x1-x0+1<52) { x0-=10; x1+=10; } /* fragment? expand frame */
+    if (x1-x0+1<62) { x0-=5;  x1+=5; }
+    if (y1-y0+1<10) { y0-= 4; y1+= 4; } /* fragment? */
+    if (x0<0) x0=0;  if (x1>=b->x) x1=b->x-1; 
+    if (y0<0) y0=0;  if (y1>=b->y) y1=b->y-1; 
+    dx=x1-x0+1;
+    dy=y1-y0+1; yy0=y0;
+    fprintf(stderr,"\n# show box + environment");
+    fprintf(stderr,"\n# show box     x= %4d %4d d= %3d %3d r= %d %d",
+	  px->x0, px->y0, px->x1 - px->x0 + 1, px->y1 - px->y0 + 1,
+	  px->x - px->x0, px->y - px->y0);
+    if (px->num_ac){ /* output table of chars and its probabilities */
+      fprintf(stderr,"\n# list box char: ");
+      for(i=0;i<px->num_ac && i<NumAlt;i++)
+      /* output the (xml-)string (picture position, barcodes, glyphs, ...) */
+        if (px->tas[i])
+         fprintf(stderr," %s(%d)",       px->tas[i]       ,px->wac[i]);
+        else
+         fprintf(stderr," %s(%d)",decode(px->tac[i],ASCII),px->wac[i]);
+    }
+    fprintf(stderr,"\n");
+    if (px->dots && px->m2 && px->m1<y0) { yy0=px->m1; dy=px->y1-yy0+1; }
+  }
+  tx=dx/80+1;
+  ty=dy/40+1; // step, usually 1, but greater on large maps 
+  fprintf(stderr,"# show pattern x= %4d %4d d= %3d %3d t= %d %d\n",
+                 x0,y0,dx,dy,tx,ty);
+  if (dx>0)
+  for(y=yy0;y<yy0+dy;y+=ty) { /* reduce the output to max 78x40 */
+
+    /* image is the boxframe + environment in the original bitmap */
+    for(x=x0;x<x0+dx;x+=tx){  /* by merging sub-pixels */
+      c1='.';
+      for(y2=y;y2<y+ty && y2<y0+dy;y2++) /* sub-pixels */
+      for(x2=x;x2<x+tx && x2<x0+dx;x2++)
+        { if((pixel(b,x2,y2)<cs)) c1='#'; }
+      // show pixels outside the box thinner/weaker
+      if (x+tx-1 < px->x0 || x > px->x1
+       || y+ty-1 < px->y0 || y > px->y1) c1=((c1=='#')?'O':',');
+      fprintf(stderr,"%c", c1 );
+    }
+
+    c1=c2=' ';
+    /* mark lines with < */
+    if (px) if (y==px->m1 || y==px->m2 || y==px->m3 || y==px->m4)  c1='<';
+    if (y==px->y0 || y==px->y1)  c2='-';  /* boxmarks */
+    fprintf(stderr,"%c%c\n",c1,c2);
+  }
+}
+
+
 /*
 // second variant, for database (with slightly other behaviour)
 // new variant
@@ -251,81 +336,91 @@ int distance2( pix *p1, struct box *box1,
 }
 
 wchar_t ocr_db(struct box *box1) {
-  int dd = 1000, j, dist = 1000;
+  int dd = 1000, dist = 1000;
   wchar_t c = UNKNOWN;
-  char buf[40], *s2;
+  char buf[200], cmd[4];
   Box *box2, *box3;
   
   if (!list_empty(&JOB->tmp.dblist)){ 
     box3 = (Box *)list_get_header(&JOB->tmp.dblist);
     if(JOB->cfg.verbose)
-      fprintf(stderr,"\n DEBUG: ocr_db");
+      fprintf(stderr,"\n#DEBUG: ocr_db (%d,%d) ",box1->x0, box1->y0);
 
     for_each_data(&JOB->tmp.dblist) {
       box2 = (Box *)list_get_current(&JOB->tmp.dblist);
       /* do preselect!!! distance() slowly */
       dd = distance2( box2->p, box2, box1->p, box1, JOB->cfg.cs);
-      if (dd < dist) {
+      if (dd <= dist) {  /* new best fit */ 
         dist = dd;
-        c = box2->c;
-        box3 = box2;
+        box3 = box2; /* box3 is a pointer and not copied box2 */
+
+        if (dist<100 && 100-dist > JOB->cfg.certainty) {
+          /* some deviation of the pattern is tolerated */
+          int i, wa;
+          for (i=0;i<box3->num_ac;i++) {
+           wa = (100-dist)*box3->wac[i]/100; /* weight *= (100-dist) */
+           if (box3->tas[i]) setas(box1,box3->tas[i],wa);
+           else              setac(box1,box3->tac[i],wa);
+          }
+          if (box3->num_ac) c=box3->tac[0]; /* 0 for strings (!UNKNOWN) */
+          if (JOB->cfg.verbose)
+            fprintf(stderr, " dist=%4d c= %c 0x%02x %s  wc= %3d", dist,
+               ((box3->c>32 && box3->c<127) ? (char) box3->c : '.'),
+               (int)box3->c, ((box3->tas[0])?box3->tas[0]:""), box3->wac[0]);
+        }
+        if (dd<=0 && ((box3->num_ac && box3->tas[0]) || box3->c >= 128
+                       || !strchr ("l1|I0O", box3->c)))
+          break; /* speedup if found */
       }
-      if (dd<=0) break; /* speedup if found */
     } end_for_each(&JOB->tmp.dblist);
-    if (JOB->cfg.verbose)
-      fprintf(stderr, " dist=%4d c= %c 0x%02x %s", dist,
-         ((c>32 && c<127)?(char)c:'~'), (int)c, ((box3->obj)?box3->obj:""));
-    if (dist > 5) { /* 5% deviation of the pattern is tolerated */
-      setac(box1, c, 100-dist);
-      c = UNKNOWN;
-    }
-    else {
-      if (c && c!=UNKNOWN)
-        setac(box1, c, 100-dist);
-      if (box3->obj) {
-        /* its not a good solution, but better then nothing: */
-        if (box1->obj) free( box1->obj);
-        box1->obj = (char *) malloc( strlen(box3->obj)+1 );
-        if (box1->obj)
-          memcpy( box1->obj, box3->obj, strlen(box3->obj)+1 );
-      }
-    }
+    
   }
 
   if( (JOB->cfg.mode&128) != 0 && c == UNKNOWN ) for(;;) { // prompt user
     // should the output go to stdout or stderr or special pipe???
-    out_x(box1);
-    fprintf(stderr,"The upper char was not recognized. Enter correct ASCII char, \"string\" or 4 to 8 digit hex unicode: ");
-    buf[0]=0; j=0;
-    fgets(buf,40,stdin); /* including \n */
-    dd=strlen(buf); if(dd>0 && buf[dd-1]=='\n') { dd--;buf[dd]=0; }
-    if ( dd>=4 && dd<=8 ) j=strtol(buf,&s2,16);
-    if( ((dd>1 && dd<4) || dd>8) || (dd>4 && (s2-buf<4 || s2-buf>8)) ){
-      if (dd<2 || buf[0]!='"' || buf[dd-1]!='"') {
-        fprintf(stderr,"invalid input\n");
-        continue;
+    int utf8_ok=0;  /* trigger this flag if input is ok */
+    out_env(box1);  /* old: out_x(box1); */
+    fprintf(stderr," The upper pattern was not recognized.\n"
+                   " Enter correct UTF-8 char or string for it: ");
+    /* UTF-8 (man 7 utf-8):
+     *   7bit = 0xxxxxxx                            (0000-007F)
+     *  11bit = 110xxxxx 10xxxxxx                   (0080-07FF)
+     *  16bit = 1110xxxx 10xxxxxx 10xxxxxx          (0800-FFFF)
+     *  21bit = 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
+     *  26bit = 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
+     *  31bit = 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
+     */
+    buf[0]=0;
+    fgets(buf,200,stdin); /* including \n */
+    dd=strlen(buf); if (dd>0 && buf[dd-1]=='\n') { dd--; buf[dd]=0; }
+    if (dd==1 && !(buf[0]&128)) {  c=buf[0]; utf8_ok=1; }
+    if (dd>1 && dd<7) {
+      int i, u0, u1; /* define UTF8-start sequences, u0=0bits u1=1bits */
+      u0=       1<<(7-dd);
+      u1=255&~((1<<(8-dd))-1);
+      for (i=1;i<dd;i++) if ((buf[i]&0xc0)!=0x80) break; /* 10xxxxxx */
+      if (i==dd && (buf[0]&(u0|u1))==u1) { utf8_ok=1;
+        c=buf[0]&(u0-1);                                 /* 11..0x.. */
+        for (i=1;i<dd;i++) { c<<=6; c|=buf[i]&0x3F; }    /* 10xxxxxx */
       }
     }
-    if( dd==1 || (dd>=4 && dd<=8) || (dd>2 && buf[0]=='"' && buf[dd-1]=='"')){
-      if (dd>1 && buf[0]=='"' && buf[dd-1]=='"') {
-        /* store a combination of chars, a string (new) */
-        c=PICTURE; /* probably a bad idea? */
-        s2=(char *)malloc(dd);
-        if (s2) { memcpy(s2,buf+1,dd-2); s2[dd-2]='\0'; }
-        box1->obj=s2;
-      } else {   /* store a single char */
-        if(dd==1) c=255&(wchar_t)buf[0]; else c=j;
+    if (dd>0){
+      if (utf8_ok==1) { setac(box1, c, 100); } /* store single char */
+      if (utf8_ok==0) {    /* store a combination of chars (string) */
+        c='_'; /* probably a bad idea? */
+        setas(box1, buf, 100);
       }
       fprintf(stderr," Store the pattern? 0=no 1=into_memory 2=to_database: ");
-      fgets(buf,40,stdin);
-      if( buf[0]>='1' && buf[0]<='2' ){
-        box1->c=c;
-        box1->num_ac=0;
+      fgets(cmd,4,stdin);
+      if( cmd[0]>='1' && cmd[0]<='2' ){
         list_app(&JOB->tmp.dblist, box1);	// append to list
       }
-      if( buf[0]=='2' ){
+      if( cmd[0]=='2' ){
         store_db(box1);
       }
+      if (JOB->cfg.verbose)
+        fprintf(stderr, "\n# got  char= %c  16bit= 0x%04x  string= \"%s\"\n",
+           ((c>32 && c<127)?(char)c:'.'), (int)c, buf);
     }
     break;
   }
