@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2006 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+#include <ctype.h>   // toupper, tolower
 #include "pgm2asc.h"
 #include "gocr.h"
 
@@ -32,10 +34,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    todo: look for thickest line, and divide if thickness=2*mean_thickness 
    Set these elements of the box structs:
 
-   m1 <-- top of upper case letters
-   m2 <-- top of small letters
+   m1 <-- top of upper case letters and (bdfhkl) (can differ)
+   m2 <-- top of letters  (acegmnopqrsuvwxyz)
    m3 <-- baseline
-   m4 <-- bottom of hanging letters
+   m4 <-- bottom of hanging letters (gqpy)
 
   performance can be improved by working with a temporary
   list of boxes of the special text line
@@ -46,6 +48,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  ToDo:
   - generate lists of boxes per line (faster access)
   - use statistics
+  - for each box look at it neighbours and set box-m1..m4
+  - m[1..4].max .min if m4.min-m3.max<1 probability lower
  */
 int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
 {
@@ -61,25 +65,25 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       lines->m1[0] = 0;
       lines->x0[0] = p->x;  /* expand to left end during detection */
       lines->x1[0] = 0;	    /* expand to right end */
+      lines->pitch[0] = JOB->cfg.spc; /* default word pitch */
+      lines->mono[0]  = 0;            /* default spacing = prop */
       lines->num++;
   }
   i = lines->num;
-  // fprintf(stderr," detect_lines1(%d %d %d %d)\n",x0,y0,dx,dy);
-  if (dy < 5)
+  if (dy < 4)
     return 0;  /* image is to low for latin chars */
   my = jj = 0;
   // get the mean value of height
   for_each_data(&(JOB->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
-    if (box2->c != PICTURE)
-      if (box2->y1 - box2->y0 > 4) {
-	if (box2->y0 >= y0 && box2->y1 <= y0 + dy
-	 && box2->x0 >= x0 && box2->x1 <= x0 + dx ) {
-	    jj++;
-	    my += box2->y1 - box2->y0 + 1;
-        }
-      }
-   } end_for_each(&(JOB->res.boxlist));
+    if ( box2->c != PICTURE
+      && box2->y1 - box2->y0 + 1 >= 4   /* 4x6 font */
+      && box2->y0 >= y0 && box2->y1 <= y0 + dy
+      && box2->x0 >= x0 && box2->x1 <= x0 + dx ) {
+	jj++;
+	my += box2->y1 - box2->y0 + 1;
+    }
+  } end_for_each(&(JOB->res.boxlist));
   if (jj == 0)
     return 0;  /* no chars detected */
     
@@ -87,10 +91,15 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
   /* ToDo: a better way could be to mark good boxes (of typical high a-zA-Z0-9)
    *    first and handle only marked boxes for line scan, exclude ?!,.:;etc
    *    but without setect the chars itself (using good statistics)
+   *    see adjust_text_lines()
    */
   my /= jj;          /* we only care about chars with high arround my */
+  if (JOB->cfg.verbose & 16)
+    fprintf(stderr," detect_lines1(%d %d %d %d) chars=%d my=%d\n#  ",
+      x0, y0, dx, dy, jj, my);
+  // "my" is the average over the whole image (bad, if different fontsizes)
 
-  if (my < 3)
+  if (my < 4)
     return 0;       /* mean high is to small => error */
 
   m4pre=m3pre=y0;   /* lower bond of upper line */
@@ -100,92 +109,114 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     // look for max. of upper and lower bound of next line
     m1 = y0 + dy;
     jj = 0;
+#if 1
+    /* this is only for test runs */
+    if (JOB->cfg.verbose & 16)
+      fprintf(stderr,"searching new line %d\n#  ",i /* lines->num */);
+#endif
 
     // find highest point of next line => store to m1-min (m1>=y)
+    // only objects greater 2/3*my and smaller 3*my are allowed
+    // a higher "!" at end of line can result in a to low m1
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+      if (box2->line>0 || box2->c == PICTURE) continue;
       yy = lines->dy * box2->x0 / (p->x); /* correct crooked lines */
       if (   box2->y0 >= y + yy && box2->y1 < y0 + dy	// lower than y
-	  && box2->x0 >= x0 && box2->x1 < x0 + dx	// in box ?
+	  && box2->x0 >= x0     && box2->x1 < x0 + dx	// in image box ?
 	  && box2->c != PICTURE	// no picture
-	  && 3 * (box2->y1 - box2->y0) > 2 * my	// right size ?
-	  && (box2->y1 - box2->y0) < 3 * my
-	  && (box2->y1 - box2->y0) > 4)
+	  && box2->num_boxes<=1 // ignore 2 for "!?i" 3 for "&auml;"
+	  && 3 * (box2->y1 - box2->y0) > 2 * my	// not to small
+	  &&     (box2->y1 - box2->y0) < 3 * my // not to big
+	  &&     (box2->y1 - box2->y0) > 4)     // minimum absolute size
        {
 	  if (box2->y0 < m1 + yy) {
 	    m1 = box2->y0 - yy;  /* highest upper boundary */
 	  }
+	  // fprintf(stderr,"\n %3d %3d %+3d %d m1= %3d",
+          //   box2->x0, box2->y0, box2->y1 - box2->y0 + 1, box2->num_boxes, m1);
        }
     } end_for_each(&(JOB->res.boxlist));
     if (m1 >= y0+dy) break; /* no further line found */
 
+    // at the moment values depend from single chars, which can 
+    //    result in bad values (ex: 4x6 /\=)
+    // ToDo: 2) mean size of next line (store list of y0,y1)
+    // ToDo: 3) count num0[(y0-m1)*16/my], num1[(y1-m1)*16/my]
+    // ToDo: or down-top search horizontal nerarest neighbours
     lines->x0[i] = x0 + dx - 1; /* expand during operation to left end */
-    lines->x1[i] = x0;
+    lines->x1[i] = x0;          /* expand to the right end of line */
     m4=m2=m1; mi=m1+my; m3=m1+2*my; jj=0;
     // find limits for upper bound, base line and ground line
     //    m2-max m3-min m4-max
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+      if (box2->line>0 || box2->c == PICTURE) continue;
+      if (   box2->y0 < y0 || box2->y1 >= y0 + dy
+	  || box2->x0 < x0 || box2->x1 >= x0 + dx ) continue; // out of image
       yy = lines->dy * box2->x0 / (p->x);
       /* check for ij-dots, used if chars of same high */
-      if (box2->y0 >= y + yy && box2->y1 < y0 + dy
-	  && box2->x0 >= x0 && box2->x1 < x0 + dx
-	  && box2->c != PICTURE	// no picture
-	  && (box2->y1 - box2->y0) < my	) {
-	if (box2->y0 >= y
-	 && box2->y1 < m1 + yy + my/4
-	 && box2->y0 < mi + yy) {
+      if ( box2->y0 >= y + yy
+	&& box2->y0 >= y
+	&& (box2->y1 - box2->y0) < my	
+	&& box2->y1 < m1 + yy + my/4
+	&& box2->y0 < mi + yy        ) {
 	    mi = box2->y0 - yy;        /* highest upper boundary i-dot */
-	}
       }
+      // fprintf(stderr,"\n check %3d %3d-%3d y=%d yy=%d m1=%d", box2->x0, box2->y0, box2->y1, y, yy, m1);
       /* get m2-max m3-min m4-max */
-      if (   box2->y0 >= y + yy && box2->y1 < y0 + dy	// lower than y
-	  && box2->x0 >= x0     && box2->x1 < x0 + dx	// in box ?
-	  && box2->x0 >= x0     && box2->x1 < x0 + dx
-	  && box2->c != PICTURE	// no picture
-	  && 3 * (box2->y1 - box2->y0) > 2 * my	// right size ?
-	  && (box2->y1 - box2->y0) < 3 * my
-	  && (box2->y1 - box2->y0) > 4)
-      if (box2->y0 >= m1                     // in m1 range?
-       && box2->y0 <= m1 + yy + 7 * my / 8   // xterm-huge-font or footnotes Dec03
-       && box2->y1 <= m1 + yy + 5 * my / 2
-       && box2->y1 >= m1 + yy + my / 2)
-       {
+      if ( box2->y0 >= y + yy 			// lower than y
+	&& 3 * (box2->y1 - box2->y0 + 1) > 2 * my  // right size ?
+	&&     (box2->y1 - box2->y0 + 1) < 3 * my  // font mix, size = 2.6*my
+	&& (box2->y1 - box2->y0 + 1) > 3           // 4x6 lowercase=4
+        && box2->y0 >= m1                     // in m1 range?
+        && box2->y0 <= m1 + yy + 9 * my / 8   // my can be to small if mixed
+        // ToDo: we need a better (local?) algorithm for big headlines > 2*my
+        && box2->y1 <= m1 + yy + 3 * my
+        && box2->y1 >= m1 + yy + my / 2
+         )
+      {
           jj++;  // count chars for debugging purpose
 	  if (box2->y0 > m2 + yy) {
 	    m2 = box2->y0 - yy;  /* highest upper boundary */
 	  }
-	  if (box2->y1 > m4 + yy) {
-	    m4 = box2->y1 - yy;  /* lowest lower boundary */
+	  if (box2->y1 > m4 + yy && (my>6 || box2->y1 < m3+my)) {
+	    m4 = box2->y1 - yy;  /* lowest lower boundary, small font lines can touch */
 	  }
-	  if (box2->y1 < m3 + yy
-	    && 2*box2->y1 > m2+m4+yy )  // care for TeX: \(^1\)Footnote 2003/12/07
- 	  /* "!" and "?" could cause trouble here, therfore this lines */
+	  if (   box2->y1 < m3 + yy
+	    && ( ( 2*box2->y1 > m2+  m4+yy && m2>m1)
+              || ( 4*box2->y1 > m1+3*m4+yy) ) )  // care for TeX: \(^1\)Footnote 2003
+ 	  /* "'!?" could cause trouble here, therefore this lines */
  	  /* ToDo: get_bw costs time, check pre and next */
 	  if( get_bw(box2->x0,box2->x1,box2->y1+1   ,box2->y1+my/2,box2->p,JOB->cfg.cs,1) == 0
 	   || get_bw(box2->x0,box2->x1,box2->y1+my/2,box2->y1+my/2,box2->p,JOB->cfg.cs,1) == 1
            || num_cross(box2->x0,box2->x1,(box2->y0+box2->y1)/2,(box2->y0+box2->y1)/2,box2->p,JOB->cfg.cs)>2 )
 	  {
 	    m3 = box2->y1 - yy;  /* highest lower boundary */
+	    // printf("\n# set1 m3 m=%3d %+2d %+2d %+2d",m1,m2-m1,m3-m1,m4-m1);
+	    // out_x(box2);
 	  }
 	  if (box2->y0 + box2->y1 > 2*(m3 + yy)
 	   && box2->y1 < m4 + yy - my/4 -1
-	   && box2->y1 >= (m2 + m4)/2 )  // care for TeX: \(^1\)Footnote 2003/12/07
+	   && box2->y1 >= (m2 + m4)/2   // care for TeX: \(^1\)Footnote 2003
+	   && m2 > m1 ) // be sure to not use ', m2 must be ok
 	  {
 	    m3 = box2->y1 - yy;  /* highest lower boundary */
+	    // printf("\n# set2 m3 m=%3d %+2d %+2d %+2d",m1,m2-m1,m3-m1,m4-m1);
 	    // out_x(box2);
 	  }
           if (box2->x1>lines->x1[i]) lines->x1[i] = box2->x1; /* right end */
           if (box2->x0<lines->x0[i]) lines->x0[i] = box2->x0; /* left end */
-	  // printf(" m=%3d %3d %3d %3d yy=%3d\n",m1,m2-m1,m3-m1,m4-m1,yy);
-	}
+	  // printf(" m=%3d %+2d %+2d %+2d yy=%3d\n",m1,m2-m1,m3-m1,m4-m1,yy);
+      }
     } end_for_each(&(JOB->res.boxlist));
 
-#if 0
+#if 1
     /* this is only for test runs */
-    if (JOB->cfg.verbose)
-      fprintf(stderr," step 1 y=%4d m1=%4d m2=%4d m3=%4d m4=%4d my=%2d chars=%3d\n",
-       y,m1,m2,m3,m4,my,jj);
+    if (JOB->cfg.verbose & 16)
+      fprintf(stderr," step 1 y=%4d m= %4d %+3d %+3d %+3d"
+                     " my=%2d chars=%3d\n#  ",
+                     y, m1, m2-m1, m3-m1, m4-m1, my, jj);
 #endif
 
     if (m3 == m1)
@@ -196,6 +227,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     ma1 = ma2 = ma3 = ma4 = i1 = i2 = i3 = i4 = jj = 0;
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+      if (box2->line>0 || box2->c == PICTURE) continue;
       yy = lines->dy * box2->x0 / (p->x);
       if (box2->y0 >= y + yy && box2->y1 < y0 + dy	// lower than y
 	  && box2->x0 >= x0 && box2->x1 < x0 + dx	// in box ?
@@ -219,16 +251,18 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       }
     } end_for_each(&(JOB->res.boxlist));
 
-    if (i1)  m1 = ma1 / i1;
-    if (i2)  m2 = ma2 / i2;
-    if (i3)  m3 = ma3 / i3;
-    if (i4)  m4 = ma4 / i4;
+    if (i1)  m1 = (ma1+i1/2) / i1; /* best rounded */
+    if (i2)  m2 = (ma2+i2/2) / i2;
+    if (i3)  m3 = (ma3+i3-1) / i3; /* round up */
+    if (i4)  m4 = (ma4+i4-1) / i4;
+    // printf("\n# .. set3 m3 m=%3d %+2d %+2d %+2d",m1,m2-m1,m3-m1,m4-m1);
 
 #endif
 
     /* expand right and left end of line */
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+      if (box2->line>0 || box2->c == PICTURE) continue;
       yy = lines->dy * box2->x0 / (p->x);
       if (   box2->y0 >= y0 && box2->y1 < y0 + dy
 	  && box2->x0 >= x0 && box2->x1 < x0 + dx	// in box ?
@@ -242,11 +276,11 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
      }
     } end_for_each(&(JOB->res.boxlist));
 
-#if 0
+#if 1
     /* this is only for test runs */
-    if (JOB->cfg.verbose)
-      fprintf(stderr," step 2 y=%4d m1=%4d m2=%4d m3=%4d m4=%4d\n",
-        y,m1,m2,m3,m4);
+    if (JOB->cfg.verbose & 16)
+      fprintf(stderr," step 2 y=%4d m= %4d %+3d %+3d %+3d\n#  ",
+        y,m1,m2-m1,m3-m1,m4-m1);
 #endif
 
     if (m4 == m1) {
@@ -254,15 +288,18 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       continue;
     }
     jj=0;
+    lines->wt[i] = 100;
     if (5 * (m2 - m1 +1) < m3 - m2 || (m2 - m1) < 2) jj|=1; /* same high */
-    if (5 * (m4 - m3 +1) < m3 - m2 || (m4 - m3) < 2) jj|=2; /* same base */
+    if (5 * (m4 - m3 +1) < m3 - m2 || (m4 - m3) < 1) jj|=2; /* same base */
+    if (jj&1) lines->wt[i] = 75*lines->wt[i]/100;
+    if (jj&2) lines->wt[i] = 75*lines->wt[i]/100;
     if (jj>0 && JOB->cfg.verbose) {
-      fprintf(stderr,"# trouble on line %d:\n",i);
-      fprintf(stderr,"#  bounds: m1=%4d m2=%4d m3=%4d m4=%4d  my=%4d\n",m1,m2-m1,m3-m1,m4-m1,my);
-      fprintf(stderr,"#  counts: i1=%4d i2=%4d i3=%4d i4=%4d\n",i1,i2,i3,i4);
-      if (jj==3) fprintf(stderr,"#  all boxes of same high!\n");
-      if (jj==1) fprintf(stderr,"#  all boxes of same upper bound!\n");
-      if (jj==2) fprintf(stderr,"#  all boxes of same lower bound!\n");
+      fprintf(stderr," trouble on line %d, wt*100= %d\n",i,lines->wt[i]);
+      fprintf(stderr,"#   m= %4d %+3d %+3d %+3d\n",m1,m2-m1,m3-m1,m4-m1);
+      fprintf(stderr,"#   i=  %3d %3d %3d %3d (counts)\n",i1,i2,i3,i4);
+      if (jj==3) fprintf(stderr,"#   all boxes of same high!\n#  ");
+      if (jj==1) fprintf(stderr,"#   all boxes of same upper bound!\n#  ");
+      if (jj==2) fprintf(stderr,"#   all boxes of same lower bound!\n#  ");
      }
     /* ToDo: check for dots ij,. to get the missing information */
 #if 1
@@ -282,8 +319,9 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       m2 = m1 + 2;                      /* font hight < 8 pixel ? */
     if (jj&2)
       m4 = m3 + (m4 - m1) / 4 + 1;	/* chars have same lower base */
-    if (jj>0 && JOB->cfg.verbose) {
-      fprintf(stderr,"#     new: m1=%4d m2=%4d m3=%4d m4=%4d  my=%4d\n",m1,m2-m1,m3-m1,m4-m1,my);
+    if (jj>0 && JOB->cfg.verbose & 16) {
+      fprintf(stderr,"#     new: m=%4d %+2d %+2d %+2d  my=%4d\n#  ",
+              m1, m2-m1, m3-m1, m4-m1, my);
      }
 #endif
 
@@ -293,9 +331,11 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       lines->m3[i] = m3;
       lines->m2[i] = m2;
       lines->m1[i] = m1;
+      lines->pitch[i] = JOB->cfg.spc; /* default word pitch */
+      lines->pitch[i] = 0;            /* default spacing */
       if (JOB->cfg.verbose & 16)
-	fprintf(stderr, "\n# line= %3d m= %4d %2d %2d %2d",
-		i, m1, m2 - m1, m3 - m1, m4 - m1);
+	fprintf(stderr, " m= %4d %+3d %+3d %+3d w= %d (line=%d)\n#  ",
+		m1, m2 - m1, m3 - m1, m4 - m1, lines->wt[i], i);
       if (i < MAXlines && m4 - m1 > 4)
 	i++;
       if (i >= MAXlines) {
@@ -309,7 +349,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
   }
   lines->num = i;
   if (JOB->cfg.verbose)
-    fprintf(stderr, " - lines= %d", lines->num-1);
+    fprintf(stderr, " num_lines= %d", lines->num-1);
   return 0;
 }
 
@@ -453,29 +493,10 @@ int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r){
  */
 }
 
-/*
- * mark longest line which was used to estimate the rotation angle
- *    ---- only for debugging purpurses ----
- */
-int mark_rotation_angle(pix *pp, int y2){
-  pix p=(*pp);
-  int x,y,dy;
-  dy=JOB->res.lines.dy;
-  if(JOB->cfg.verbose&32){
-    for(y=0;y<p.y;y++)
-      for(x=0;x<p.x;x++)
-        JOB->tmp.ppo.p[x+p.x*y]=p.p[x+p.x*y]&(192);
-    for(x=0;x<JOB->tmp.ppo.x;x++)
-      if((x&35)>32)
-        put(&JOB->tmp.ppo,x,y2+dy*x/JOB->tmp.ppo.x,255,32);
-    // writebmp("out10.bmp",JOB->tmp.ppo,JOB->cfg.verbose); // colored should be better
-  }
-  return 0;
-}
-
-/* ToDo: Does someone know about a faster square root for integer?
- *       without using math-lib
- *   "Intervalschachtelung" ToDo: translate to engl.
+/* ToDo: herons algorithm for square root x=(x+y/x)/2 is more efficient
+ *       than interval subdivision (?) (germ.: Intervallschachtelung)
+ *       without using matlib
+ *  see http://www.math.vt.edu/people/brown/doc/sqrts.pdf
  */
 int my_sqrt(int x){
   int y0=0,y1=x,ym;
@@ -512,8 +533,8 @@ int new_detect_rotation_angle(job_t *job, int *dx, int *dy){
       x2 = (box2->x0 + box2->x1)/2;
       y2 = (box2->y0 + box2->y1)/2; /* get middle point of the box */
       /* subfunction probability of char */
-      if  (box2->x1 - box2->x0 < 6) continue;
-      if  (box2->y1 - box2->y0 < 6) continue;
+      if  (box2->x1 - box2->x0 < 3) continue; /* smallest font is 4x6 */
+      if  (box2->y1 - box2->y0 < 5) continue;
       /* set maximum possible distance */
       mindist = job->src.p.x * job->src.p.x + job->src.p.y * job->src.p.y;
       box_nn=NULL;
@@ -580,7 +601,7 @@ int new_detect_rotation_angle(job_t *job, int *dx, int *dy){
       fprintf(stderr,"# rotation angle (x,y,num) (%d,%d,%d) (%d,%d,%d), pass %d\n",
               dx1,dy1,nn1,dx2,dy2,nn2,pass);
   }
-  if (nn2>4 && abs(dy2*100/dx2)>50)
+  if (nn2>4 && abs(dy2*100)>abs(dx2*50))
     fprintf(stderr,"<!-- gocr will fail, strong rotation angle detected -->\n");
   return (*dy)*((*dx)?job->src.p.x/(*dx):1);
   /*
@@ -625,7 +646,7 @@ int detect_rotation_angle(job_t *job){
     } end_for_each(&(job->res.boxlist));
     if ( 2*x > pp->x && pp->x > 100 ){
      if(JOB->cfg.verbose)fprintf(stderr, "# perfect flat line found, dy=0\n");
-     mark_rotation_angle(pp, y2);
+     JOB->res.lines.ly=y2;
      return 0;
     }
     b3=b4=NULL;
@@ -668,8 +689,8 @@ int detect_rotation_angle(job_t *job){
       }
     }
     JOB->res.lines.dy=dy;
+    JOB->res.lines.ly=y2;
     if(JOB->cfg.verbose)fprintf(stderr," - at crosses=%3d dy=%d\n",i,dy);
-    mark_rotation_angle( pp, y2);
   }
   return 0;
 }
@@ -678,41 +699,152 @@ int detect_rotation_angle(job_t *job){
 int detect_text_lines(pix * pp, int mo) {
 
   if (JOB->cfg.verbose)
-    fprintf(stderr, "# scanning lines ");	// or read from outside???
+    fprintf(stderr, "# scanning lines\n# ... ");	// or read from outside???
   if (mo & 4){
     if (JOB->cfg.verbose) fprintf(stderr, "(zoning) ");
     detect_lines2(pp, 0, 0, pp->x, pp->y, 0);	// later replaced by better algo
   } else
     detect_lines1(pp, 0, 0, pp->x, pp->y);	// old algo
 
-  if (JOB->cfg.verbose & 32) {	      // debug-output preparations
-    int i, x, y, dy;
-    dy = JOB->res.lines.dy;      /* can also be done in write_img !? */
-
-    for (i = 0; i < JOB->res.lines.num; i++) { // mark lines
-      for (x = JOB->res.lines.x0[i]; x < JOB->res.lines.x1[i]; x++) {
-        y = JOB->res.lines.m1[i];
-        if ((x & 7) == 4)    
-          put(&JOB->tmp.ppo, x, y + dy * x / JOB->tmp.ppo.x, 255, 32);
-        y = JOB->res.lines.m2[i];
-        if ((x & 3) == 2)    
-          put(&JOB->tmp.ppo, x, y + dy * x / JOB->tmp.ppo.x, 255, 32);
-        y = JOB->res.lines.m3[i];
-        if ((x & 1) == 1)    
-          put(&JOB->tmp.ppo, x, y + dy * x / JOB->tmp.ppo.x, 255, 32);
-        y = JOB->res.lines.m4[i];
-        if ((x & 7) == 4)    
-          put(&JOB->tmp.ppo, x, y + dy * x / JOB->tmp.ppo.x, 255, 32);
-      }
-    }
-    //writebmp("out10.bmp",JOB->tmp.ppo,JOB->cfg.verbose); 
-    // colored should be better
-  }
-
   if(JOB->cfg.verbose) fprintf(stderr,"\n");
   return 0;
 }
 
+
+/* ----- adjust lines --------------- */
+// rotation angle? JOB->res.lines.dy, .x0  removed later
+// this is for cases, where m1..m4 is not very sure detected before 
+//  chars are recognized
+int adjust_text_lines(pix * pp, int mo) {
+  struct box *box2;
+  int *m, /* summ m1..m4, num_chars for m1..m4, min m1..m4, max. m1..m4 */
+      l, i, dy, dx, diff=0, y0, y1;
+
+  if ((l=JOB->res.lines.num)<2) return 0;  // ???
+  if (JOB->cfg.verbose)
+    fprintf(stderr, "# adjust lines ");
+  m=(int *)malloc(l*16*sizeof(int));
+  if (!m) { fprintf(stderr," malloc failed\n"); return 0;}
+  for (i=0;i<16*l;i++) m[i]=0; /* initialize */
+  dy=JOB->res.lines.dy;  /* tan(alpha) of skewing */
+  dx=JOB->src.p.x;       /* width of image */
+  // js: later skewing is replaced by one transformation of vectorized image 
+
+  for_each_data(&(JOB->res.boxlist)) {
+    box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+    if (box2->line<=0) continue;
+    if (box2->num_ac<1) continue;
+    if (box2->wac[0]<95) continue;
+    y0=box2->y0-((box2->x1)*dy/dx); /* corrected by page skewing */
+    y1=box2->y1-((box2->x1)*dy/dx);
+    if (strchr("aemnr",(char)box2->tac[0])) {  // cC vV sS oO ... is unsure!
+      m[box2->line*16+1]+=y0; m[box2->line*16+5]++; // num m2
+      m[box2->line*16+2]+=y1; m[box2->line*16+6]++; // num m3
+      if (m[box2->line*16+ 9]>y0) m[box2->line*16+ 9]=y0; /* min m2 */
+      if (m[box2->line*16+13]<y0) m[box2->line*16+13]=y0; /* max m2 */
+      if (m[box2->line*16+10]>y1) m[box2->line*16+10]=y1; /* min m3 */
+      if (m[box2->line*16+14]<y1) m[box2->line*16+14]=y1; /* max m3 */
+    }
+    if (strchr("bdhklABDEFGHIKLMNRT123456789",(char)box2->tac[0])) {
+      m[box2->line*16+0]+=y0; m[box2->line*16+4]++; // num m1
+      m[box2->line*16+2]+=y1; m[box2->line*16+6]++; // num m3
+      if (m[box2->line*16+ 8]>y0) m[box2->line*16+ 8]=y0; /* min m1 */
+      if (m[box2->line*16+12]<y0) m[box2->line*16+12]=y0; /* max m1 */
+      if (m[box2->line*16+10]>y1) m[box2->line*16+10]=y1; /* min m3 */
+      if (m[box2->line*16+14]<y1) m[box2->line*16+14]=y1; /* max m3 */
+    }
+    if (strchr("gq",(char)box2->tac[0])) {
+      m[box2->line*16+1]+=y0; m[box2->line*16+5]++; // num m2
+      m[box2->line*16+3]+=y1; m[box2->line*16+7]++; // num m4
+      if (m[box2->line*16+ 9]>y0) m[box2->line*16+ 9]=y0; /* min m2 */
+      if (m[box2->line*16+13]<y0) m[box2->line*16+13]=y0; /* max m2 */
+      if (m[box2->line*16+11]>y1) m[box2->line*16+11]=y1; /* min m4 */
+      if (m[box2->line*16+15]<y1) m[box2->line*16+15]=y1; /* max m4 */
+    }
+  } end_for_each(&(JOB->res.boxlist));
+  
+  diff=0;
+  for (i=1;i<l;i++) {
+    if (m[i*16+4]) diff+=abs(JOB->res.lines.m1[i]-m[i*16+0]/m[i*16+4]);
+    if (m[i*16+5]) diff+=abs(JOB->res.lines.m2[i]-m[i*16+1]/m[i*16+5]);
+    if (m[i*16+6]) diff+=abs(JOB->res.lines.m3[i]-m[i*16+2]/m[i*16+6]);
+    if (m[i*16+7]) diff+=abs(JOB->res.lines.m4[i]-m[i*16+3]/m[i*16+7]);
+    /* recalculate sureness, empirically */
+    if (m[i*16+4]*m[i*16+5]*m[i*16+6]*m[i*16+7] > 0)
+      JOB->res.lines.wt[i]=(JOB->res.lines.wt[i]+100)/2;
+    else 
+      JOB->res.lines.wt[i]=(JOB->res.lines.wt[i]*90)/100;
+    // set mean values of sure detected bounds (rounded precisely)
+    if ( m[i*16+4]) JOB->res.lines.m1[i]=(m[i*16+0]+m[i*16+4]/2)/m[i*16+4];
+    if ( m[i*16+5]) JOB->res.lines.m2[i]=(m[i*16+1]+m[i*16+5]/2)/m[i*16+5];
+    if ( m[i*16+6]) JOB->res.lines.m3[i]=(m[i*16+2]+m[i*16+6]/2)/m[i*16+6];
+    if ( m[i*16+7]) JOB->res.lines.m4[i]=(m[i*16+3]+m[i*16+7]/2)/m[i*16+7];
+    // care about very small fonts
+    if (JOB->res.lines.m2[i]-JOB->res.lines.m1[i]<=1 && m[i*16+5]==0 && m[i*16+4])
+        JOB->res.lines.m2[i]=JOB->res.lines.m1[i]+2;
+    if (JOB->res.lines.m2[i]-JOB->res.lines.m1[i]<=1 && m[i*16+4]==0 && m[i*16+5])
+        JOB->res.lines.m1[i]=JOB->res.lines.m2[i]-2;
+    if (JOB->res.lines.m4[i]-JOB->res.lines.m3[i]<=1 && m[i*16+7]==0 && m[i*16+6])
+        JOB->res.lines.m4[i]=JOB->res.lines.m3[i]+2;
+    if (JOB->res.lines.m4[i]-JOB->res.lines.m3[i]<=1 && m[i*16+6]==0 && m[i*16+7])
+        JOB->res.lines.m3[i]=JOB->res.lines.m4[i]-2;
+    if ( m[i*16+7]<1 &&
+        JOB->res.lines.m4[i]
+      <=JOB->res.lines.m3[i]+(JOB->res.lines.m3[i]-JOB->res.lines.m2[i])/4 )
+        JOB->res.lines.m4[i]=
+        JOB->res.lines.m3[i]+(JOB->res.lines.m3[i]-JOB->res.lines.m2[i])/4;
+    if ( m[i*16+7]<1 && m[i*16+12+2]>0 &&   // m4 < max.m3+..
+        JOB->res.lines.m4[i] < 2*m[i*16+12+2]-JOB->res.lines.m3[i]+2 )
+        JOB->res.lines.m4[i] = 2*m[i*16+12+2]-JOB->res.lines.m3[i]+2;
+    if (JOB->res.lines.m4[i]<=JOB->res.lines.m3[i])
+        JOB->res.lines.m4[i]= JOB->res.lines.m3[i]+1; /* 4x6 */
+                   
+    if (JOB->cfg.verbose & 16)
+      fprintf(stderr, "\n#  line= %3d m= %4d %+3d %+3d %+3d "
+                         " n= %2d %2d %2d %2d  w= %3d diff= %d",
+	i, JOB->res.lines.m1[i],
+	   JOB->res.lines.m2[i] - JOB->res.lines.m1[i],
+	   JOB->res.lines.m3[i] - JOB->res.lines.m1[i],
+	   JOB->res.lines.m4[i] - JOB->res.lines.m1[i],
+	   m[i*16+4],m[i*16+5],m[i*16+6],m[i*16+7],
+	   JOB->res.lines.wt[i], diff);
+  }
+  diff=0; // count adjusted chars
+#if 1
+  for_each_data(&(JOB->res.boxlist)) {
+    box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+    if (box2->line<=0) continue;
+    /* check if box was on the wrong line, ToDo: search a better line */
+    if (2*box2->y0<2*JOB->res.lines.m1[box2->line]
+                    -JOB->res.lines.m4[box2->line]
+                    +JOB->res.lines.m1[box2->line]) box2->line=0;
+    if (2*box2->y1>2*JOB->res.lines.m4[box2->line]
+                    +JOB->res.lines.m4[box2->line]
+                    -JOB->res.lines.m1[box2->line]) box2->line=0;
+    /* do adjustments */
+    if (box2->num_ac>0
+     && strchr("cCoOpPsSuUvVwWxXyYzZ",(char)box2->tac[0])) {
+      if (box2->y0-((box2->x1)*dy/dx)
+       < (JOB->res.lines.m1[box2->line]+JOB->res.lines.m2[box2->line])/2
+      && islower(box2->tac[0])
+      ) { setac(box2,toupper(box2->tac[0]),(box2->wac[0]+101)/2); diff++; }
+      if (box2->y0-((box2->x1)*dy/dx)
+       > (JOB->res.lines.m1[box2->line]+JOB->res.lines.m2[box2->line]+1)/2
+      && isupper(box2->tac[0])
+      ){ setac(box2,tolower(box2->tac[0]),(box2->wac[0]+101)/2); diff++; }
+    }
+    box2->m1=JOB->res.lines.m1[box2->line]+((box2->x1)*dy/dx);
+    box2->m2=JOB->res.lines.m2[box2->line]+((box2->x1)*dy/dx);
+    box2->m3=JOB->res.lines.m3[box2->line]+((box2->x1)*dy/dx);
+    box2->m4=JOB->res.lines.m4[box2->line]+((box2->x1)*dy/dx);
+  } end_for_each(&(JOB->res.boxlist));
+#endif
+
+  free(m);
+  if(JOB->cfg.verbose) fprintf(stderr," changed= %d\n",diff);
+  return(diff);
+}
+
 /* ---- measure mean character
  * recalculate mean width and high after changes in boxlist
  * ToDo: only within a Range?
@@ -744,8 +876,8 @@ int calc_average() {
     }
   } end_for_each(&(JOB->res.boxlist));
   if ( JOB->res.numC ) {		/* avoid div 0 */
-    JOB->res.avY = JOB->res.sumY / JOB->res.numC;
-    JOB->res.avX = JOB->res.sumX / JOB->res.numC;
+    JOB->res.avY = (JOB->res.sumY+JOB->res.numC/2) / JOB->res.numC;
+    JOB->res.avX = (JOB->res.sumX+JOB->res.numC/2) / JOB->res.numC;
   }
   if (JOB->cfg.verbose){
     fprintf(stderr, "# averages: mXmY= %d %d nC= %d n= %d\n",
@@ -761,25 +893,29 @@ int detect_pictures(job_t *job) {
   int i = 0, x0, y0, x1, y1, num_h;
   struct box *box2, *box4;
 
-  if ( JOB->res.numC == 0 ) return -1;
-  JOB->res.avY = JOB->res.sumY / JOB->res.numC;
-  JOB->res.avX = JOB->res.sumX / JOB->res.numC;
-  if (JOB->cfg.verbose)
+  if ( job->res.numC == 0 ) {
+    if (job->cfg.verbose) fprintf(stderr, "# detect pictures, numC=0 ???\n");
+   return -1;
+  }
+  /* ToDo: set Y to uppercase mean value? */
+  job->res.avY = (job->res.sumY+job->res.numC/2) / job->res.numC;
+  job->res.avX = (job->res.sumX+job->res.numC/2) / job->res.numC;
+  if (job->cfg.verbose)
     fprintf(stderr, "# detect pictures, frames, noAlphas, mXmY= %d %d ... ",
-  	    JOB->res.sumX/JOB->res.numC, JOB->res.sumY/JOB->res.numC);
-  for_each_data(&(JOB->res.boxlist)) {
-    box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+  	    job->res.avX, job->res.avY);
+  for_each_data(&(job->res.boxlist)) {
+    box2 = (struct box *)list_get_current(&(job->res.boxlist));
     if (box2->c == PICTURE) continue;
     x0 = box2->x0;    x1 = box2->x1;
     y0 = box2->y0;    y1 = box2->y1;
 
     /* pictures could be of unusual size */ 
-    if (x1 - x0 + 1 > 4 * JOB->res.avX || y1 - y0 + 1 > 4 * JOB->res.avY) {
+    if (x1 - x0 + 1 > 4 * job->res.avX || y1 - y0 + 1 > 4 * job->res.avY) {
       /* count objects on same baseline which could be chars */
       /* else: big headlines could be misinterpreted as pictures */
       num_h=0;
-      for_each_data(&(JOB->res.boxlist)) {
-        box4 = (struct box *)list_get_current(&(JOB->res.boxlist));
+      for_each_data(&(job->res.boxlist)) {
+        box4 = (struct box *)list_get_current(&(job->res.boxlist));
         if (box4->c == PICTURE) continue;
         if (box4->y1-box4->y0 > 2*(y1-y0)) continue;
         if (2*(box4->y1-box4->y0) < y1-y0) continue;
@@ -789,16 +925,16 @@ int detect_pictures(job_t *job) {
          || box4->y1 < y1 - (y1-y0+1)/2)  continue;
         // ToDo: continue if numcross() only 1, example: |||IIIll|||
         num_h++;
-      } end_for_each(&(JOB->res.boxlist));
+      } end_for_each(&(job->res.boxlist));
       if (num_h>4) continue;
       box2->c = PICTURE;
       i++;
     }
     /* ToDo: pictures could have low contrast=Sum((pixel(p,x,y)-160)^2) */
-  } end_for_each(&(JOB->res.boxlist));
+  } end_for_each(&(job->res.boxlist));
   // start second iteration
-  if (JOB->cfg.verbose) {
-    fprintf(stderr, " %d - boxes %d\n", i, JOB->res.numC-i);
+  if (job->cfg.verbose) {
+    fprintf(stderr, " %d - boxes %d\n", i, job->res.numC-i);
   }
   calc_average();
   return 0;
