@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2006  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,7 +29,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stdio.h>
 #include <assert.h>
 #include <string.h>
+#ifdef HAVE_GETTIMEOFDAY 
 #include <sys/time.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -40,7 +42,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "ocr0.h"		/* only_numbers */
 
 /* the following line is used to name the package (make dist) */
-#define version_string "0.39"
+#define version_string "0.41"
 
 static void out_version(int v) {
   fprintf(stderr, " Optical Character Recognition --- gocr " version_string "\n");
@@ -53,47 +55,29 @@ static void out_version(int v) {
 
 static void help(void) {
   out_version(0);
+  /* output is shortened to essentials, see manual page for details */
   fprintf(stderr,
 	  " using: gocr [options] pnm_file_name  # use - for stdin\n"
-	  " options:\n"
+	  " options (see gocr manual pages for more details):\n"
 	  " -h        - get this help\n"
 	  " -i name   - input image file (pnm,pgm,pbm,ppm,pcx,...)\n"
-	  " -i -      - read PNM from stdin (djpeg -gray a.jpg | gocr -)\n"
 	  " -o name   - output file  (redirection of stdout)\n"
 	  " -e name   - logging file (redirection of stderr)\n"
-#ifdef __USE_POSIX
-	  " -x name   - progress output (file, fifo, or fd=1..255)\n"
-#else
-	  " -x name   - progress output (file or fifo)\n"
-#endif
-	  " -p name   - database path (including final slash, default is ./db/)\n"
+	  " -x name   - progress output to fifo (see manual)\n"
+	  " -p name   - database path including final slash (default is ./db/)\n"
 	  " -f fmt    - output format (ISO8859_1 TeX HTML XML UTF8 ASCII)\n"
 	  " -l num    - threshold grey level 0<160<=255 (0 = autodetect)\n"
-	  " -d num    - dust_size (remove smaller clusters, -1 = autodetect)\n"
+	  " -d num    - dust_size (remove small clusters, -1 = autodetect)\n"
 	  " -s num    - spacewidth/dots (0 = autodetect)\n"
-	  " -v num    - verbose  [summed]\n"
-	  "      1      print more info\n"
-	  "      2      list shapes  of boxes (see -c)\n"
-	  "      4      list pattern of boxes (see -c)\n"
-	  "      8      print pattern after recognition\n"
-	  "     16      print line infos\n"
-	  "     32      debug outXX.pgm\n"
-	  " -c string - list of chars (_ = not recognized chars, debug)\n"
+	  " -v num    - verbose (see manual page)\n"
+	  " -c string - list of chars (debugging, see manual)\n"
 	  " -C string - char filter (ex. hexdigits: ""0-9A-Fx"", only ASCII)\n"
-	  " -m num    - operation modes, ~ = switch off\n"
-	  "      2      use database (early development)\n"
-	  "      4      layout analysis, zoning (development)\n"
-	  "      8      ~ compare non recognized chars\n"
-	  "     16      ~ divide overlapping chars\n"
-	  "     32      ~ context correction\n"
-	  "     64      char packing (development)\n"
-	  "    130      extend database, prompts user (128+2, early development)\n"
-	  "    256      switch off the OCR engine (makes sense together with -m 2)\n"
-	  " -n   1      only numbers\n"
+	  " -m num    - operation modes (bitpattern, see manual)\n"
+	  " -a num      value of certainty (in percent, 0..100, default=95)\n"
 	  " examples:\n"
-	  "\tgocr -v 33 text1.pbm                # some infos + out30.bmp\n"
-	  "\tgocr -v 7 -c _YV text1.pbm          # list unknown, Y and V chars\n"
-	  "\tdjpeg -pnm -gray text.jpg | gocr -  # use jpeg-file via pipe\n"
+	  "\tgocr -m 4 text1.pbm                   # do layout analyzis\n"
+	  "\tgocr -m 130 -p ./database/ text1.pbm  # extend database\n"
+	  "\tdjpeg -pnm -gray text.jpg | gocr -    # use jpeg-file via pipe\n"
 	  "\n");
   exit(0);
 }
@@ -229,6 +213,10 @@ static void process_arguments(job_t *job, int argn, char *argv[])
 	ini_progress(s1);
 	i++;
 	break;
+      case 'a': /* set certainty */
+	job->cfg.certainty = atoi(s1);;
+	i++;
+	break;
       default:
 	fprintf(stderr, "# unknown option use -h for help\n");
       }
@@ -281,11 +269,7 @@ static void read_picture(job_t *job) {
   if (strstr(job->src.fname, ".pcx"))
     readpcx(job->src.fname, &job->src.p, job->cfg.verbose);
   else
-#ifdef HAVE_PNM_H
-    readpnm(job->src.fname, &job->src.p, job->cfg.verbose);
-#else
     readpgm(job->src.fname, &job->src.p, job->cfg.verbose);
-#endif
 }
 
 // subject of change, we need more output for XML (ToDo)
@@ -300,9 +284,8 @@ void print_output(job_t *job) {
   while (line) {
     // notice: decode() is shiftet to getTextLine since 0.38
     fputs(line, stdout);
-    if (job->cfg.out_format==HTML
-     || job->cfg.out_format==XML) fputs("<br />",stdout);
-    fputc('\n', stdout);
+    if (job->cfg.out_format==HTML) fputs("<br />",stdout);
+    if (job->cfg.out_format!=XML)  fputc('\n', stdout);
     line = getTextLine(linecounter++);
   }
   free_textlines();
