@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2005 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,7 +22,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
  - look for ??? for preliminary code
 */
-
+ 
 /* General headerfile with gocr-definitions */
 
 #ifndef __GOCR_H__
@@ -62,30 +62,36 @@ typedef enum direction DIRECTION;
 
 #define MAXlines 1024
 
-/* FIXME jb: remove tline, not used
-struct tline { 
-   int x0,y0,x1,y1; 	// middle points
-};
-*/
-
+/* ToDo: if we have a tree instead of a list, a line could be a node object */
 struct tlines {
     int num;
     /* ToDo: add dx, so we have a vector (dx,dy) (js) */
+    int ly;             /* longest line, set by detect_rotation_angle */
     int dy;		/* offset for straight/skew lines dy(x+px) */
     int m1[MAXlines],   /* start of line = upper bound of 'A' */
         m2[MAXlines],   /* upper bound of 'e' */
         m3[MAXlines],	/* lower bound of 'e' = baseline */
         m4[MAXlines];	/* stop of line = lower bound of 'q' */
     int x0[MAXlines],
-        x1[MAXlines];		/* left and right border */
+        x1[MAXlines];	/* left and right border */
+    int wt[MAXlines];   /* weight, how sure thats correct in percent, v0.41 */
+    int pitch[MAXlines]; /* word pitch (later per box?), v0.41 */
+    int mono[MAXlines];  /* spacing type, 0=proportional, 1=monospaced */
 };
 
 #define NumAlt 10 // maximal number of alternative chars (table length)
+#define MaxNumFrames 8       // maximum number of frames per char/box
+#define MaxFrameVectors 128  // maximum vectors per frame (*8=1KB/box)
+/* ToDo: use only malloc_box(),free_box(),copybox() for creation, destroy etc.
+ *       adding reference_counter to avoid pointer pointing to freed box
+ */
 struct box {	// this structure should contain all pixel infos of a letter
-    int x0,x1,y0,y1,x,y,dots;	// xmin,xmax,ymin,ymax,reference-pixel,i-dots
-    wchar_t c,ac;	// detected (alternate) char
-    wchar_t modifier;	// default=0, see compose() in unicode.c
-    int num;		// same nummer = same char
+    int x0,x1,y0,y1,x,y,dots;	 // xmin,xmax,ymin,ymax,reference-pixel,i-dots
+    int num_boxes, // 1 "abc", 2 "!i?", 3 "&auml;" (composed objects) 0.41
+        num_subboxes;   // 1 for "abdegopqADOPQR", 2 for "B"  (holes) 0.41
+    wchar_t c;		// detected char (same as tac[0], obsolete?)
+    wchar_t modifier;	// default=0, see compose() in unicode.c 
+    int num;		// same number = same char
     int line;		// line number (points to struct tlines lines)
     int m1,m2,m3,m4;	// m2 = upper boundary, m3 = baseline
     // planed: sizeof hole_1, hole_2, certainty (run1=100%,run2=90%,etc.)
@@ -94,71 +100,90 @@ struct box {	// this structure should contain all pixel infos of a letter
     int num_ac;         // length of table (alternative chars), default=0
     wchar_t tac[NumAlt]; // alternative chars, only used by setac(),getac()
     int     wac[NumAlt]; // weight of alternative chars
-    char    *obj;        // pointer to text-object ... (default=NULL)
+    char   *tas[NumAlt]; // alternative UTF8-strings or XML codes if tac[]=0
+                         //   replacing old obj
+	                 // ToDo: (*obj)[NumAlt] + olen[NumAlt] ???
+	                 // ToDo: bitmap for possible Picture|Object|Char ???
+//  char    *obj;        // pointer to text-object ... -> replaced by tas[]
                          //  ... (melted chars, barcode, picture coords, ...)
                          //  must be freed before box is freed!
-                         //  do _not_ copy obj !!!!!
+                         //  do _not_ copy only the pointer to object
+    /* --------------------------------------------------------
+     *  extension since v0.41 js05, Store frame vectors,
+     *  which is a table of vectors sourrounding the char and its
+     *  inner white holes. The advantage is the independence from
+     *  resolution, handling of holes, overlap and rotation.
+     * --------------------------------------------------------- */
+    int num_frames;      // number of frames: 1 for cfhklmnrstuvwxyz
+                         //                   2 for abdegijopq
+    int num_frame_vectors[MaxNumFrames]; // index to next frame
+                // biggest frame should be stored first (outer frame)
+                // biggest has the maximum pair distance
+                // num vector loops
+    int frame_vector[MaxFrameVectors][2]; // may be 16*int=fixpoint_number
+    
 };
 typedef struct box Box;
 
-#define outbounds(p, tx, ty)   	    (tx < 0 || ty < 0 || tx >= (p)->x || ty >= (p)->y)
+/* true if the coordination pair (a,b) is outside the image p */
+#define outbounds(p, a, b)  (a < 0 || b < 0 || a >= (p)->x || b >= (p)->y)
 
+/* ToDo: this structure seems to be obsolete, remove it */
 typedef struct path {
-  int start;			// color at the beginning of the path
-				// (0=white, 1=black)
-  int *x;			// x coordinates of transitions
-  int *y;			// y coordinates of transitions
-  int num;			// current number of entries in x or y
-  int max;			// maximum number of entries in x or y
-				// (if more values need to be stored,
-				// the arrays are enlarged)
+  int start;	// color at the beginning of the path, (0=white, 1=black)
+  int *x;	// x coordinates of transitions
+  int *y;	// y coordinates of transitions
+  int num;	// current number of entries in x or y
+  int max;	// maximum number of entries in x or y
+  // (if more values need to be stored, the arrays are enlarged)
 } path_t;
 
 /* job_t contains all information needed for an OCR task */
 typedef struct job_s {
-  struct { /* source data */
+  struct {       /* source data */
     char *fname; /* input filename; default value: "-" */
-    pix p; /* source pixel data */
-           /* pixelmap 8bit gray */
+    pix p;       /* source pixel data, pixelmap 8bit gray */
   } src;
   struct { /* temporary stuff, e.g. buffers */
 #ifdef HAVE_GETTIMEOFDAY
     struct timeval init_time; /* starting time of this job */
 #endif
-    pix ppo; /* pixmap for visual debugging output */
+    pix ppo; /* pixmap for visual debugging output, obsolete */
 
     /* sometimes recognition function is called again and again, if result was 0
        n_run tells the pixel function to return alternative results */
-    int n_run; /* num of run, if run_2 critical pattern get other results */
-               /* used for 2nd try, pixel uses slower filter function etc. */
-    List dblist; /* FIXME jb: desc */
+    int n_run;   /* num of run, if run_2 critical pattern get other results */
+                 /* used for 2nd try, pixel uses slower filter function etc. */
+    List dblist; /* list of boxes loaded from the character database */
   } tmp;
-  struct { /* results */
-    List boxlist; /* FIXME jb: desc */
-    List linelist; /* FIXME jb: desc */
+  struct {         /* results */
+    List boxlist;  /* store every object in a box, which contains */
+                   /* the characteristics of the object (see struct box) */
+    List linelist; /* recognized text lines after recognition */
     
-    struct tlines lines; /* used to access to line-data */
+    struct tlines lines; /* used to access to line-data (statistics) */
                          /* here the positions (frames) of lines are */
                          /* stored for further use */
-
-    int avX,avY; /* average X,Y (avX=sumX/numC) */
-    int sumX,sumY,numC; /* sum of all X,Y; num chars */
+    int avX,avY;         /* average X,Y (avX=sumX/numC) */
+    int sumX,sumY,numC;  /* sum of all X,Y; num chars */
   } res;
-  struct { /* configuration */
-    int cs; /* FIXME jb: grey level 0<160<=255; default value: 0 */
-            /* critical value (pixel<cs => black pixel) */
-
-    int spc; /* spacewidth/dots (0 = autodetect); default value: 0 */
+  struct {    /* configuration */
+    int cs;   /* critical grey value (pixel<cs => black pixel) */
+              /* range: 0..255,  0 means autodetection */
+    int spc;  /* spacewidth/dots (0 = autodetect); default value: 0 */
     int mode; /* operation modes; default value: 0 */
-            /* operation mode (see --help) */
-    int dust_size; /* dust size; default value: 10 */
+              /* operation mode (see --help) */
+    int dust_size;    /* dust size; default value: 10 */
     int only_numbers; /* numbers only; default value: 0 */
     int verbose; /* verbose mode; default value: 0 */ 
                  /* verbose option (see --help) */
     FORMAT out_format; /* output format; default value: ISO8859_1*/
-    char *lc; /* list of chars (_ = not recognized chars); default value: "_" */
+    char *lc; /* debuglist of chars (_ = not recognized chars) */
+              /* default value: "_" */
     char *db_path; /* pathname for database; default value: NULL */
-    char *cfilter; /* char filter; default value: NULL */
+    char *cfilter; /* char filter; default value: NULL, ex: "A-Za-z" */
+        /* limit of certainty where chars are accepted as identified */
+    int  certainty; /* in units of 100 (percent); 0..100; default 95 */
   } cfg;
 } job_t;
 
@@ -171,10 +196,11 @@ void job_free(job_t *job);
 //FIXME jb: remove JOB;
 extern job_t *JOB;
 
-// kalkulate the overlapp of the line (0-1) with black points 
+// calculate the overlapp of the line (0-1) with black points 
 // by rekursiv bisection 
 // (evl. Fehlertoleranz mit pixel in Umgebung dx,dy suchen) (umschaltbar) ???
 // MidPoint Line Algorithm (Bresenham) Foley: ComputerGraphics better?
+// will be replaced by vector functions
 
 // gerade y=dy/dx*x+b, implizit d=F(x,y)=dy*x-dx*y+b*dx=0 
 // incrementell y(i+1)=m*(x(i)+1)+b, F(x+1,y+1)=f(F(x,y))
