@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2006  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -63,38 +63,15 @@ void list_init( List *l ) {
   if ( !l )
      return;
 
-  l->header = l->tail = NULL;
+  l->start.next     = &l->stop;
+  l->stop.previous  = &l->start;
+  l->start.previous = l->stop.next = NULL;
+  l->start.data = l->stop.data = NULL;
   l->current = NULL;
-  l->fix = NULL;
   l->level = -1;
   l->n = 0;
 }
 
-/* appends data to the list. Returns 1 on error, 0 if OK. */
-int list_app( List *l, void *data ) {
-  Element *e;
-  
-  if ( !l || !data )
-     return 1;
-  if ( !(e = (Element *)malloc(sizeof(Element))) )
-    return 1;
-  
-  e->data = data;
-  if ( !l->header ) {
-    l->header = l->tail = e;
-    l->n = 1;
-    e->previous = e->next = NULL;
-    return 0;
-  }
-
-  l->tail->next = e;
-  e->previous = l->tail;
-  l->tail = e;
-  e->next = NULL;
-  l->n++;
-  return 0;
-}
-
 /* inserts data before data_after. If data_after == NULL, appends.
    Returns 1 on error, 0 if OK. */
 int list_ins( List *l, void *data_after, void *data) {
@@ -104,44 +81,58 @@ int list_ins( List *l, void *data_after, void *data) {
   if ( !l || !data )
     return 1;
 
-  if ( !data_after || !l->header )
+  if ( !data_after || !l->n )
     return list_app(l, data);
 
-  /* alloc a new element */
-  if( !(e = (Element *)malloc(sizeof(Element))) )
-    return 1;
-  e->data = data;
-
   /* get data_after element */
   if ( !(after_element = list_element_from_data(l, data_after)) )
     return 1;
 
-  e->next = after_element;
+  /* alloc a new element */
+  if( !(e = (Element *)malloc(sizeof(Element))) )
+    return 1;
+  e->data     = data;
+  e->next     = after_element;
   e->previous = after_element->previous;
-  if ( after_element->previous ) /* i.e., if after_element != list->header */
-    after_element->previous->next = e;
-  else /* update list->header */
-    l->header = e;
-  after_element->previous = e;
+  after_element->previous->next = e;
+  after_element->previous       = e;
   l->n++;
 
   return 0;
 }
 
+/* appends data to the list. Returns 1 on error, 0 if OK. */
+/* same as list_ins(l,NULL,data) ??? */
+int list_app( List *l, void *data ) {
+  Element *e;
+  
+  if ( !l || !data )
+     return 1;
+  if ( !(e = (Element *)malloc(sizeof(Element))) )
+    return 1;
+  
+  e->data     = data;
+  e->previous = l->stop.previous;
+  e->next     = l->stop.previous->next;
+  l->stop.previous->next = e;
+  l->stop.previous       = e;
+  l->n++;
+  return 0;
+}
+
 /* returns element associated with data. */
 Element *list_element_from_data( List *l, void *data ) {
   Element *temp;
 
-  if ( !l || !data )
+  if ( !l || !data || !l->n)
     return NULL;
 
-  if ( !(temp = l->header) )
-    return NULL;
+  temp = l->start.next;
 
   while ( temp->data != data ) {
-    temp = temp->next;
-    if ( !temp )
+    if ( !temp || temp==&l->stop )
       return NULL;
+    temp = temp->next;
   }
   return temp;
 }
@@ -149,11 +140,14 @@ Element *list_element_from_data( List *l, void *data ) {
 /* deletes (first) element with data from list. User must free data.
    Returns 0 if OK, 1 on error.
    This is the internal version, that shouldn't be called usually. Use the
-   list_del() macro instead.*/
+   list_del() macro instead.
+ */
 int list_del( List *l, void *data ) {
   Element *temp;
   int i;
 
+  if (!data) return 1; /* do not delete start or stop element */
+
   /* find element associated with data */
   if ( !(temp = list_element_from_data(l, data)) )
     return 1;
@@ -161,31 +155,19 @@ int list_del( List *l, void *data ) {
   /* test if the deleted node is current in some nested loop, and fix it. */
   for ( i = l->level; i >= 0; i-- ) {
     if ( l->current[i] == temp ) {
-      l->fix[i] = temp;
+      l->current[i] = temp->previous;
     }
   }
 
-  /* fix previous */
-  if ( temp == l->header ) {
-    l->header = temp->next;
-    if (l->header)
-      l->header->previous = NULL;
-  }
-  else
-    temp->previous->next = temp->next;
-
-  /* fix next */
-  if ( temp == l->tail ) {
-    l->tail = temp->previous;
-    if (l->tail)
-      l->tail->next = NULL;
-  }
-  else {
-    if (temp->next)
-      temp->next->previous = temp->previous;
-  }
+  temp->previous->next = temp->next;
+  temp->next->previous = temp->previous;
+  temp->previous = temp->next = NULL; /* mark as freed */
+/*
+  fprintf(stderr,"\n# list_del=%p start=%p stop=%p",temp,&l->start,&l->stop);
+*/
 
   /* and free stuff */
+  free(temp); /* element pointing to data, fixed mem-leak 0.41 */
   l->n--;
   return 0;
 }
@@ -194,9 +176,7 @@ int list_del( List *l, void *data ) {
 void list_free( List *l ) {
   Element *temp, *temp2;
 
-  if ( !l )
-    return;
-  if ( !l->header )
+  if ( !l || !l->n )
     return;
 
   if ( l->current ) {
@@ -204,46 +184,37 @@ void list_free( List *l ) {
   }
   l->current = NULL;
 
-  if ( l->fix ) {
-    free(l->fix);
-  }
-  l->fix = NULL;
-
-  temp = l->header;
-  while ( temp ) {
+  temp = l->start.next;
+  while ( temp && temp!=&l->stop) {
     temp2 = temp->next;
     free(temp);
     temp = temp2;
   }
-  l->header = l->tail = NULL;
+  l->start.next    = &l->stop;
+  l->stop.previous = &l->start;
 }
 
 /* setup a new level of for_each */
 int list_higher_level( List *l ) {
   Element **newcur;
-  Element **newfix;
   
-  if ( !l || !l->header ) 
-    return 1;
+  if ( !l ) return(1);
 
   /*
      Security-check: NULL pointer passed to realloc.
       ANSI allows this, but it may cause portability problems.
   */    
   newcur = (Element **)realloc(l->current, (l->level+2)*sizeof(Element *));
-  newfix = (Element **)realloc(l->fix    , (l->level+2)*sizeof(Element *));
-  if ( !newcur || !newfix ) { /* doesn't blow everything */
-    fprintf(stderr, " realloc failed! level++=%d current[]=%p fix[]=%p\n", 
-	l->level, l->current, l->fix);
-    return 1;
+  if (newcur) {
+    l->current = newcur;
+    l->level++;
+    l->current[l->level] = l->start.next;
+  }
+  g_debug(fprintf(stderr, " level++=%d current[]=%p\n",
+     l->level, l->current);)
+  if ( !newcur  ) {
+    fprintf(stderr, " realloc failed! abort\n"); return(1);
   }
-  l->current = newcur;
-  l->fix = newfix;
-  l->level++;
-  l->current[l->level] = l->header;
-  l->fix[l->level] = NULL;
-  g_debug(fprintf(stderr, " level++=%d current[]=%p fix[]=%p\n", l->level,
-      l->current, l->fix);)
   return 0;
 }
 
@@ -253,16 +224,13 @@ void list_lower_level( List *l ) {
 
   if (!l->level) {
     free(l->current); /* calm -lefence */
-    free(l->fix    );
     l->current = NULL; /* could be important */
-    l->fix     = NULL;
   } else {
     l->current = (Element **)realloc(l->current, l->level*sizeof(Element *));
-    l->fix     = (Element **)realloc(l->fix    , l->level*sizeof(Element *));
   }
   l->level--;
-  g_debug(fprintf(stderr, " level--=%d current[]=%p fix[]=%p\n", l->level, 
-      l->current, l->fix);)
+  g_debug(fprintf(stderr, " level--=%d current[]=%p\n", l->level, 
+      l->current);)
 }
 
 /* returns the next item data */
@@ -299,26 +267,20 @@ void list_sort( List *l, int (*compare)(const void *, const void *) ) {
     return;
 
   for (i = 0; i < l->n; i++ ) {
-    for ( temp = l->header->next; temp != NULL; temp = temp->next ) {
-      if ( compare((const void *)temp->previous->data, (const void *)temp->data) > 0 ) {
+    for ( temp = l->start.next->next;
+          temp != NULL && temp != &l->stop; temp = temp->next ) {
+      if ( temp->previous == &l->start ) continue;
+      if ( compare((const void *)temp->previous->data,
+                   (const void *)temp->data) > 0 ) {
 
    	/* swap with the previous node */
 	prev = temp->previous;
-	
-	if ( prev->previous )
-    	  prev->previous->next = temp;
-  	else /* update header */
-	  l->header = temp;
-
+  	prev->previous->next = temp;
+        temp->next->previous = prev;
 	temp->previous = prev->previous;
+	prev->next     = temp->next;
 	prev->previous = temp;
-	prev->next = temp->next;
-	if ( temp->next )
-	  temp->next->previous = prev;
-	else /* update tail */
-	  l->tail = prev;
-	temp->next = prev;
-
+	temp->next     = prev;
 	/* and make sure the node in the for loop is correct */
 	temp = prev;
 
