@@ -40,9 +40,8 @@ struct element {
 typedef struct element Element;
 
 struct list {
-   Element *header;
-   Element *tail;
-   Element **fix;		/* fixes the list_del header problem */
+   Element start;               /* simplifies for(each_element) { ... */
+   Element stop;                /*   ... list_del() ... }  v0.41      */
    Element **current;	 	/* for(each_element) */
    int n;			/* number of elements */
    int level;			/* level of nested fors */
@@ -66,9 +65,9 @@ void *	list_next		( List *l, void *data );
 void *	list_prev		( List *l, void *data );
 void	list_sort		( List *l, int (*compare)(const void *, const void *) );
 
-#define list_empty(l)			((l)->header == NULL ? 1 : 0)
-#define list_get_header(l)		((l)->header->data)
-#define list_get_tail(l)		((l)->tail->data)
+#define list_empty(l)			((l)->start.next == &(l)->stop ? 1 : 0)
+#define list_get_header(l)		((l)->start.next->data)
+#define list_get_tail(l)		((l)->stop.previous->data)
 #define list_get_current(l)		((l)->current[(l)->level]->data)
 #define list_get_cur_prev(l)		((l)->current[(l)->level]->previous == NULL ? \
 			NULL : (l)->current[(l)->level]->previous->data )
@@ -78,20 +77,10 @@ void	list_sort		( List *l, int (*compare)(const void *, const void *) );
 
 #define for_each_data(l)		\
  if (list_higher_level(l) == 0) { \
-   for ( ; (l)->current[(l)->level]; (l)->current[(l)->level] = \
-	(l)->current[(l)->level]->next ) { \
-     if ( (l)->fix[(l)->level] ) { /* fix level */\
-       int i; \
-       for ( i = (l)->level - 1; i >= 0; i-- ) {  \
-       /* check if some other copy of (l)->fix[(l)->level] exists */ \
-        \
-         if ( (l)->fix[i] == (l)->fix[(l)->level] ) break; \
-       } \
-       if ( i < 0 ) { /* no, it doesn't. Free it */ \
-         free((l)->fix[(l)->level]); \
-       } \
-       (l)->fix[(l)->level] = NULL; \
-     }
+   for ( ; (l)->current[(l)->level] \
+        && (l)->current[(l)->level]!=&(l)->stop; (l)->current[(l)->level] = \
+	(l)->current[(l)->level]->next ) {
+
 
 #define end_for_each(l)			\
    } \
