@@ -7,7 +7,7 @@
      in ocr0_shared structure.
  *  ------------------------------------------------------------  */
 
-typedef struct ocr0_shared {  /* ToDo: implement it */
+typedef struct ocr0_shared {  /* shared variables and properties */
 
   struct box *box1;  /* box in whole image */
   pix *bp;           /* extracted temporarly box, cleaned */
@@ -17,6 +17,7 @@ typedef struct ocr0_shared {  /* ToDo: implement it */
   int x0, x1, y0, y1; /* box coordinates related to box1 */
   int dx, dy;         /* size of box */
   int hchar, gchar;   /* relation to m1..m4 */
+  int aa[4][4];       /* corner points, see xX (x,y,dist^2,vector_idx) v0.41 */
   holes_t holes;      /* list of holes (max MAX_HOLES) */
 
 } ocr0_shared_t;
