@@ -74,6 +74,8 @@ void readpcx(char *name,pix *p,int vvv){	 	// see pcx.format.txt
 // ------------------------------------------------------------------------
 
 // write bmp 8bit palette no RLE 
+//   bit 2+3 used for color coding (markers)
+//   replaced by writeppm (ppm.gz) and is obsolate now, removed later
 void writebmp(char *name,pix p,int vvv){ 	// see pcx.format.txt
   int nx,ny,i,y,rest[4]={0,0,0,0};
   FILE *f1;  
@@ -81,8 +83,9 @@ void writebmp(char *name,pix p,int vvv){ 	// see pcx.format.txt
   long fs,fo,hs,is;	// filesize, offset, headersize, imagesize
 
   nx=p.x; ny=p.y; pic=p.p;
+  if (nx&3) nx+=4-(nx&3);  // must be mod4 ?
   hs=40;	    // bmi headersize fix
-  is=((nx+3)&3)*ny; // imagesize
+  is=nx*ny; // imagesize
   fo=14+hs+4*256;
   fs=fo+is;
   for(i=0;i<54;i++){ h[i]=0; }
@@ -111,18 +114,18 @@ void writebmp(char *name,pix p,int vvv){ 	// see pcx.format.txt
   h[50]=0;h[51]=1;	// colorimportand (0=all)
   // 54 - endofheader
   for(i=0;i<256;i++){ 
-     h[54+4*i+0]=((~((i & 16)*8)) &                    (i & (128+64)))|63;
-     h[54+4*i+1]=((~((i & 16)*8)) &  (~((i & 32)*4)) & (i & (128+64)))|63;
-     h[54+4*i+2]=(  ((i & 16)*4)  | ((~((i & 32)*4)) & (i & (128+64)))|63);
+     h[54+4*i+0]=((~((i & 2)*64)) &                    (i & (128+64)))|63;
+     h[54+4*i+1]=((~((i & 2)*64)) &  (~((i & 4)*32)) & (i & (128+64)))|63;
+     h[54+4*i+2]=(  ((i & 2)* 8)  | ((~((i & 4)*32)) & (i & (128+64)))|63);
   }			 // blue-green-red
   f1=fopen(name,"wb"); if(!f1) fprintf(stderr," error opening file\n");
   if(!f1)ERR("open");		// open-error
   if(fwrite(h,1,54+4*256,f1)!=54+4*256)ERR("write head"); 
   if(vvv) fprintf(stderr,"# write BMP x=%d y=%d\n",nx,ny);
   for(y=ny-1;y>=0;y--){
-    if(((int)fwrite(pic+nx*y,1,nx,f1))!=nx)ERR("write");
-    if(nx&3)
-    if(((int)fwrite(rest,1,4-(nx&3),f1))!=4-(nx&3))ERR("write");  // must be mod4
+    if(((int)fwrite(pic+p.x*y,1,p.x,f1))!=p.x)ERR("write");
+    if(nx>p.x)
+    if(((int)fwrite(rest,1,nx-p.x,f1))!=nx-p.x)ERR("write");
   }
   fclose(f1);
 }
