@@ -488,7 +488,10 @@ int pixel(pix *p, int x, int y){
   if ( x < 0 || y < 0 || x >= p->x || y >= p->y ) 
     return 255 & ~7;
 
-  if (JOB->tmp.n_run & 2) { /* use the filters (corection of errors) */
+  /* filter will be used only once later, when vectorization replaces pixel
+   * processing 
+   */
+  if (JOB->tmp.n_run > 0) { /* use the filters (correction of errors) */
 #if FILTER_METHOD == FILTER_BY_NUMBER
     int pix = pixel_filter_by_number(p, x, y);
 #ifdef FILTER_CHECKED
@@ -524,9 +527,6 @@ int pixel(pix *p, int x, int y){
 #endif /* FILTER_BY_NUMBER */
   }
 
-  if ((JOB->tmp.n_run&1) && x > 0 && y > 0 && x+1 < p->x && y+1 < p->y){ 
-    return pixel_faxfilter(p, x, y);
-  }
   return (pixel_atp(p,x,y) & ~7);
 }
 
