@@ -1,22 +1,22 @@
-/* (c) Joerg Schulenburg 2000
+/* (c) Joerg Schulenburg 2000-2006
     v0.1.0 initial version (stdin added)
     v0.2.0 popen added
     v0.2.7 review by Bruno Barberi Gnecco
     v0.39  autoconf
+    v0.41  fix integer and heap overflow, change color output
  */
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+/* #include <unistd.h> */
+#endif
 
 #include "pnm.h"
-#ifdef HAVE_PNM_H
 #ifdef HAVE_PAM_H
 # include <pam.h>
-#endif
-# include <pnm.h>
 # include <sys/types.h>
 # include <sys/stat.h>
 # include <fcntl.h>
@@ -36,16 +36,21 @@
  * 
  * The below values have been chosen to reflect the fact that paper
  * goes a reddish-yellow as it ages.
+ *
+ * v0.41: for better performance, we use integer instead of double
+ *        this integer value divided by 1024 (2^10) gives the factor 
  */
-#define PPM_RED_WEIGHT .499
-#define PPM_GREEN_WEIGHT .387
-#define PPM_BLUE_WEIGHT .114
+#define PPM_RED_WEIGHT   511  /* .499 */
+#define PPM_GREEN_WEIGHT 396  /* .387 */
+#define PPM_BLUE_WEIGHT  117  /* .114 */
 
 /*
     feel free to expand this list of usable converting programs
     Note 1: the last field must be NULL.
     Note 2: "smaller" extensions must come later: ".pnm.gz" must come
        before ".pnm".
+    calling external programs is a security risk
+    ToDo: for better security replace gzip by /usr/bin/gzip !
  */
 char *xlist[]={
   ".pnm.gz",	"gzip -cd",  /* compressed pnm-files, gzip package */
@@ -64,6 +69,7 @@ char *xlist[]={
   ".png",	"pngtopnm", /* Portable Network Graphics (PNG) format */
   ".ps",	"pstopnm -stdout -portrait -pgm", /* postscript */
   ".eps",	"pstopnm -stdout -portrait -pgm", /* encapsulated postscript */
+   /* gs -sDEVICE=pgmraw -sOutputFile=- -g609x235 -r141x141 -q -dNOPAUSE */
   ".fig",	"fig2dev -L ppm -m 3", /* xfig files, transfig package */
   NULL
 };
@@ -80,24 +86,32 @@ char *testsuffix(char *name){
   return NULL;
 }
 
-#ifdef HAVE_PNM_H
-/* js: I need pgm format output from pnm input
-       This routine fails if input is pbm.
-        Bruno, could this be changed?
+
+char read_char(FILE *f1){	// filter #-comments
+  char c;
+  int  m;
+  for(m=0;;){
+    c=fgetc(f1);
+    if( feof(f1)   )		E0("read feof");
+    if( ferror(f1) )		F0("read ferror");
+    if( c == '#'  )		{ m = 1; continue; }
+    if( m ==  0   )		return c;
+    if( c == '\n' )		m = 0;
+  }
+}
+
+
+/*
+   for simplicity only PAM of netpbm is used, the older formats
+   PBM, PGM and PPM can be handled implicitly by PAM routines (js05)
  */
-void readpnm(char *name, pix * p, int vvv) {
-  FILE *fp;
-  char *pip;
-  int i, j;
-  int ppm_format = 0;
 #ifdef HAVE_PAM_H
+void readpgm(char *name, pix * p, int vvv) {
+  FILE *fp;
+  char *pip, magic1, magic2;
+  int i, j, sample, minv = 0, maxv = 0;
   struct pam inpam;
   tuple *tuplerow;
-#else
-  xel *data;
-  xelval maxval;
-  int format;
-#endif
 
   assert(p);
 
@@ -113,13 +127,14 @@ void readpnm(char *name, pix * p, int vvv) {
 	F1("opening file %s", name);
     }
     else {
-      char *buf = (char *)malloc((strlen(pip)+strlen(name)+2)*
-	  sizeof(char));
+      char *buf = (char *)malloc((strlen(pip)+strlen(name)+2));
       sprintf(buf, "%s %s", pip, name);
       if (vvv) {
 	fprintf(stderr, "# popen( %s )\n", buf);
       }
 #ifdef HAVE_POPEN
+      /* potential security vulnerability, if name contains tricks */
+      /* example: gunzip -c dummy | rm -rf * */
       fp = popen(buf, "r");
 #else
       F0("sorry, compile with HAVE_POPEN to use pipes");
@@ -131,83 +146,82 @@ void readpnm(char *name, pix * p, int vvv) {
   }
 
   /* read pgm */
-#ifdef HAVE_PAM_H
   pnm_readpaminit(fp, &inpam, sizeof(inpam));
 
   p->x = inpam.width;
   p->y = inpam.height;
+  magic1=(inpam.format >> 8) & 255; /* 'P' for PNM,PAM */
+  magic2=(inpam.format     ) & 255; /* '7' for PAM */
+  minv=inpam.maxval;
+  if (vvv) {
+      fprintf(stderr, "# readpam: format=0x%04x=%c%c h*w(d*b)=%d*%d(%d*%d)\n",
+        inpam.format, /* magic1*256+magic2 */
+        ((magic1>31 && magic1<127)?magic1:'.'),
+        ((magic2>31 && magic2<127)?magic2:'.'),
+        inpam.height,
+        inpam.width,
+        inpam.depth,
+        inpam.bytes_per_sample);
+  }
+  if ( (1.*(p->x*p->y))!=((1.*p->x)*p->y) )
+    F0("Error integer overflow");
   if ( !(p->p = (unsigned char *)malloc(p->x*p->y)) )
     F1("Error at malloc: p->p: %d bytes", p->x*p->y);
   tuplerow = pnm_allocpamrow(&inpam);
-  if (PAM_FORMAT_TYPE(inpam.format) == PPM_FORMAT) {
-    if (vvv) {
-      fprintf(stderr, "# ppm file detected\n");
-    }
-    ppm_format = 1;
-  }
   for ( i=0; i < inpam.height; i++ ) {
     pnm_readpamrow(&inpam, tuplerow);
     for ( j = 0; j < inpam.width; j++ ) {
-      if (ppm_format)
-        p->p[i*inpam.width+j] 
-          = PPM_RED_WEIGHT * tuplerow[j][0]
-          + PPM_GREEN_WEIGHT * tuplerow[j][1]
-          + PPM_BLUE_WEIGHT * tuplerow[j][2];
+      if (inpam.depth>=3)
+        /* tuplerow is unsigned long (see pam.h sample) */
+        /* we expect 8bit or 16bit integers,
+           no overflow up to 32-10-2=20 bits */
+        sample
+          = ((PPM_RED_WEIGHT   * tuplerow[j][0] + 511)>>10)
+          + ((PPM_GREEN_WEIGHT * tuplerow[j][1] + 511)>>10)
+          + ((PPM_BLUE_WEIGHT  * tuplerow[j][2] + 511)>>10);
       else
-        p->p[i*inpam.width+j] = tuplerow[j][0];
+        sample = tuplerow[j][0];
+      sample = 255 * sample / inpam.maxval; /* normalize to 8 bit */
+      p->p[i*inpam.width+j] = sample;
+      if (maxv<sample) maxv=sample;
+      if (minv>sample) minv=sample;
     }
   }
+  if (vvv) fprintf(stderr,"# readpam: min=%d max=%d\n",minv,maxv);
   pnm_freepamrow(tuplerow);
-#else
-  pnm_readpnminit(fp, &(p->x), &(p->y), &maxval, &format);
-  if ( !(p->p = (unsigned char *)malloc(p->x*p->y)) )
-    F0("Error at malloc: p->p");
-  data = pnm_allocrow(p->x);
-  if (PNM_FORMAT_TYPE(format) == PPM_FORMAT) {
-    if (vvv) {
-      fprintf(stderr, "# ppm file detected\n");
-    }
-    ppm_format = 1;
-  }
-  for ( i=0; i < p->y; i++ ) {
-    pnm_readpnmrow(fp, data, p->x, maxval, format);
-    for ( j = 0; j < p->x; j++ ) {
-      if (ppm_format)
-        p->p[i*p->x+j] 
-          = PPM_RED_WEIGHT * PPM_GETR(data[j])
-          + PPM_GREEN_WEIGHT * PPM_GETG(data[j])
-          + PPM_BLUE_WEIGHT * PPM_GETB(data[j]);
-      else
-        p->p[i*p->x+j] = PNM_GET1(data[j]);
-    }
-  }
-  pnm_freerow(data);
-#endif
   fclose(fp);
 
   p->bpp = 1;
 }
 
-#endif /* HAVE_PNM_H */
-
-char read_char(FILE *f1){	// filter #-comments
-  char c;
-  int  m;
-  for(m=0;;){
-    c=fgetc(f1);
-    if( feof(f1)   )		E0("read feof");
-    if( ferror(f1) )		F0("read ferror");
-    if( c == '#'  )		{ m = 1; continue; }
-    if( m ==  0   )		return c;
-    if( c == '\n' )		m = 0;
+#else
+/*
+   if PAM not installed, here is the fallback routine,
+   which is not so powerful but needs no dependencies from other libs
+ */
+static int fread_num(char *buf, int bps, FILE *f1) {
+  int mode, j2, j3; char c1;
+  for (j2=0;j2<bps;j2++) buf[j2]=0; // initialize value to zero
+  for(mode=0;!feof(f1);){ // mod=0 means skip leading spaces, 1 scan digits
+    c1=read_char(f1);
+    if (isspace(c1)) { if (mode==0) continue; else break; }
+    mode=1; // digits scan mode
+    if( !isdigit(c1) ) F0("unexpected char");
+    for (j3=j2=0;j2<bps;j2++) {      // multiply bps*bytes by 10
+      j3 = buf[j2]*10 + j3;  // j3 is used as result and carry
+      buf[j2]=j3 & 255; j3>>=8; 
+    }
+    buf[0] += c1-'0';
   }
+  return 0;
 }
 
 void readpgm(char *name,pix *p,int vvv){	// raw-ppm
-  char c1,c2,*pip;     int nx,ny,nc,mod,i,j,k;	// buffer
+  char c1,c2,*pip;     int nx,ny,nc,mod,i,j;	// buffer
   FILE *f1;            unsigned char *pic;
   char buf[512];
-  int lx,ly,dx;
+  int lx, ly, dx;
+  int bps=1; /* bytes per sample (0..255..65535...) */
 
   pip=NULL;
   if(name[0]=='-' && name[1]==0){
@@ -217,22 +231,24 @@ void readpgm(char *name,pix *p,int vvv){	// raw-ppm
     if(!pip){
       f1=fopen(name,"rb"); if(!f1)F1("opening file %s",name);
     } else {
-      sprintf(buf,"%s %s",pip,name); /* how to prevent OVL ? */
+      sprintf(buf,"%s %s",pip,name); /* ToDo: how to prevent OVL ? */
       if(vvv){ fprintf(stderr,"# popen( %s )\n",buf); }
 #ifdef HAVE_POPEN
       f1=popen(buf,"r");
 #else
-      F0("sorry, compile with HAVE_POPEN to use pipes");
+      F0("only PNM files supported (compiled without HAVE_POPEN)");
 #endif
       if(!f1)F1("opening pipe %s",buf);
     }
   }
-  c1=fgetc(f1); if(feof(f1))F0("read");
-  c2=fgetc(f1); if(feof(f1))F0("read");
-  if(c1!='P')F0("first byte should be P in PNM-files");
-  if(c2!='5' && c2!='4' && c2!='6')// F0("no PNM-RAW"); // PGM-RAW,PBM-RAW,PPM-RAW
-  if(c2!='1' && c2!='2' && c2!='3')F0("no PNM");
-  nx=ny=nc=0; if( c2=='4' || c2=='1') nc=1;
+  c1=fgetc(f1); if (feof(f1)) F0("read");
+  c2=fgetc(f1); if (feof(f1)) F0("read");
+  // check the first two bytes of the PNM file 
+  //         PBM   PGM   PPM
+  //    TXT   P1    P2    P3
+  //    RAW   P4    P5    P6
+  if (c1!='P' || c2 <'1' || c2 >'6') F0("no PNM (P1..P6)"); // PNM-TXT/RAW
+  nx=ny=nc=0; if (c2=='4' || c2=='1') nc=1;
   for(mod=0;((c2=='5' || c2=='2') && (mod&7)<6)
         ||  ((c2=='6' || c2=='3') && (mod&7)<6)		
         ||  ((c2=='4' || c2=='1') && (mod&7)<4);)		
@@ -250,65 +266,49 @@ void readpgm(char *name,pix *p,int vvv){	// raw-ppm
       else if(mod==5) nc=nc*10+c1-'0';
     }
   }
-  if(nc>=256)F0("nc >= 256"); // really RAW?
   if(vvv)
-  fprintf(stderr,"# PNM P%c x=%d y=%d c=%d head=%ld",c2,nx,ny,nc,ftell(f1));
+   fprintf(stderr,"# PNM P%c x=%d y=%d c=%d head=%ld",c2,nx,ny,nc,ftell(f1));
   if( c2=='4' && (nx&7)!=0 ){
     /* nx=(nx+7)&~7;*/ if(vvv)fprintf(stderr," PBM2PGM nx %d",(nx+7)&~7);
   }
+  if (nc>> 8) bps=2; // bytes per color and pixel
+  if (nc>>16) bps=3;
+  if (nc>>24) bps=4;
   fflush(stdout);
-  pic=(unsigned char *)malloc( nx*ny*((c2=='6')?3:1) );
-  if(pic==NULL)F0("memory failed");				// no memory
-  if( c2=='5' )
-    if(ny!=(int)fread(pic,   nx,ny,f1))F0("read");	// read all lines
-  if( c2=='2' )for(mod=j=i=0;i<nx*ny && !feof(f1);){		// not very clever
-    c1=read_char(f1);
-    if( !isdigit(c1) ) {  if( !isspace(c1) )F0("unexpected char");
-      if(1&mod) { pic[i]=j*255/nc; i++; j=0; mod=0; }  }
-    else { j=j*10+c1-'0'; mod=1; };
+  if ( (1.*(nx*ny))!=((1.*nx)*ny) )
+    F0("Error integer overflow");
+  pic=(unsigned char *)malloc( nx*ny );
+  if(pic==NULL)F0("memory failed");			// no memory
+  /* this is a slow but short routine for P1 to P6 formats */
+  if( c2=='5' && c2=='2' ) /* slow PGM-RAW/ASC read pixelwise */
+    for (i=0;i<nx*ny;i++) {
+     if (c2=='5') { if(bps!=(int)fread(buf,1,bps,f1))F0("read"); }
+     else for (j=0;j<3;j++) fread_num(buf+j*bps, bps, f1);
+     pic[i]=buf[bps-1]; /* store the most significant byte */
+    }
+  // we want to normalize brightness to 0..255
+  if (c2=='6' || c2=='3') {  // PPM-RAW/ASC
+    for (i=0;i<nx*ny;i++) {
+      if (c2=='6') { if (3*bps!=(int)fread(buf,1,3*bps,f1)) F0("read"); }
+      else for (j=0;j<3;j++) fread_num(buf+j*bps, bps, f1);
+      pic[i]
+          = ((PPM_RED_WEIGHT   * buf[  bps-1] + 511)>>10)
+          + ((PPM_GREEN_WEIGHT * buf[2*bps-1] + 511)>>10)
+          + ((PPM_BLUE_WEIGHT  * buf[3*bps-1] + 511)>>10);
+      /* normalized to 0..255 */
+    }
   }
-  if( c2=='1' )for(mod=j=i=0,nc=255;i<nx*ny && !feof(f1);){	// PBM-ASCII 0001100
+  if( c2=='1' )for(mod=j=i=0,nc=255;i<nx*ny && !feof(f1);){ // PBM-ASCII 0001100
     c1=read_char(f1);
     if( isdigit(c1) ) { pic[i]=((c1=='0')?255:0); i++; }
     else if( !isspace(c1) )F0("unexpected char");
   }
-  if( c2=='3' )for(mod=k=j=i=0;i<nx*ny*3 && !feof(f1);){
-    c1=read_char(f1);
-    if( !isdigit(c1) ) {  if( !isspace(c1) )F0("unexpected char");
-      if(1&mod) { k+=j; if(mod==5){ pic[i]=k/3; i++; }
-        j=0; mod=(mod+1)%6; }  }
-    else { j=j*10+c1-'0'; if(!(mod&1)) mod++; };
-  }
-  if( c2=='6' ){
-    if(ny!=(int)fread(pic, 3*nx,ny,f1))F0("read");	// read all lines
-    for(ly=0;ly<ny;ly++)
-    for(lx=0;lx<nx;lx++)
-    pic[lx+ly*nx]=( pic[0+3*lx+3*ly*nx]
-                   +pic[1+3*lx+3*ly*nx]
-                   +pic[2+3*lx+3*ly*nx] ) / 3;
-  }
   if( c2=='4' ){				// PBM-RAW
     dx=(nx+7)&~7;				// dx (mod 8)
     if(ny!=(int)fread(pic,dx>>3,ny,f1))F0("read");	// read all bytes
     for(ly=ny-1;ly>=0;ly--)
     for(lx=nx-1;lx>=0;lx--)
     pic[lx+ly*nx]=( (128 & pic[(lx+ly*dx)>>3]<<(lx & 7)) ? 0 : 255 );
-#if 0
-    // using gray values for fuzzy logic
-    for(ly=ny-2;ly>0;ly--)
-    for(lx=nx-2;lx>0;lx--){				// takes time!
-      pic[lx+ly*nx]=
-            ((pic[ lx   + ly   *nx]>128)?192:64)	// reduce contrast
-           +((pic[(lx+1)+(ly  )*nx]>128)?  8:-8)	// max 4*8+4*4=48;
-           +((pic[(lx-1)+(ly  )*nx]>128)?  8:-8)
-           +((pic[(lx  )+(ly+1)*nx]>128)?  8:-8)
-           +((pic[(lx  )+(ly-1)*nx]>128)?  8:-8)
-           +((pic[(lx+1)+(ly+1)*nx]>128)?  4:-4)
-           +((pic[(lx-1)+(ly+1)*nx]>128)?  4:-4)
-           +((pic[(lx+1)+(ly-1)*nx]>128)?  4:-4)
-           +((pic[(lx-1)+(ly-1)*nx]>128)?  4:-4);
-    }
-#endif
     nc=255;
   }
   if(name[0]=='-' && name[1]==0){
@@ -322,64 +322,8 @@ void readpgm(char *name,pix *p,int vvv){	// raw-ppm
   p->p=pic;  p->x=nx;  p->y=ny; p->bpp=1;
   if(vvv)fprintf(stderr,"\n");
 }
+#endif /* HAVE_PAM_H */
 
-#ifdef HAVE_PNM_H
-int writepgmfile(FILE *fp, pix *p) {
-  int i, j;
-  gray *data;
-
-  pgm_writepgminit(fp, p->x, p->y, PGM_MAXMAXVAL, 0);
-  data = pgm_allocrow(p->x);
-  for (i = 0; i < p->y; i++) {
-    for (j = 0; j < p->x; j++) 
-      data[j] = p->p[i*p->x+j];
-    pgm_writepgmrow(fp, data, p->x, PGM_MAXMAXVAL, 0);
-  }
-  pgm_freerow(data);
-  return 0;
-}
-
-int writepgm(char *nam, pix *p) {
-  FILE *fp;
-
-  fp = fopen(nam, "wb");
-  if (!fp)
-    F0("open");			// open-error
-  writepgmfile(fp, p);
-  fclose(fp);
-  return 0;
-}
-
-int writeppm(char *nam, pix *p) {
-  FILE *fp;char buf[1024];
-  
-  /* unsecure because possible buf overflow */
-  sprintf(buf,"pgmtoppm rgbi:/1.0/1.0/1.0 >%s",nam);
-#ifdef HAVE_POPEN
-  fp=popen(buf,"w");if(!fp)F1("open %s",buf);
-#else
-  F0("sorry, compile with HAVE_POPEN to use pipes");
-#endif
-  writepgmfile(fp, p);
-  pclose(fp);
-  return 0;
-}
-
-int writepbm(char *nam, pix *p) {
-  FILE *fp;char buf[1024];
-  
-  /* unsecure because possible buf overflow */
-  sprintf(buf,"pgmtopbm -threshold -value 0.5 >%s",nam);
-#ifdef HAVE_POPEN
-  fp=popen(buf,"w");if(!fp)F1("open %s",buf);
-#else
-  F0("sorry, compile with HAVE_POPEN to use pipes");
-#endif
-  writepgmfile(fp, p);
-  pclose(fp);
-  return 0;
-}
-#else
 int writepgm(char *nam,pix *p){// P5 raw-pgm
   FILE *f1;int a,x,y;
   f1=fopen(nam,"wb");if(!f1)F0("open");		// open-error
@@ -395,23 +339,64 @@ int writepgm(char *nam,pix *p){// P5 raw-pgm
   return 0;
 }
 
-// pgm2ppm, use last 3 bits for farbcoding 
-int writeppm(char *nam,pix *p){// P6 raw-ppm
-  FILE *f1;int x,y; unsigned char rgb[3];
-  f1=fopen(nam,"wb");if(!f1)F0("open");		// open-error
+/*
+ * pgmtoppm or pnmtopng, use last 3 bits for farbcoding 
+ * replaces old writebmp variant
+ */
+int writeppm(char *nam, pix *p){ /* P6 raw-ppm */
+  FILE *f1=NULL; int x,y,f1t=0; unsigned char rgb[3], gray, bits;
+  char buf[128];
+  if (strchr(nam,'|')) return -1;  /* no nasty code */
+  if (strstr(nam,".ppm")) { f1=fopen(nam,"wb"); }
+#ifdef HAVE_POPEN
+  /* be sure that nam contains hacker code like "dummy | rm -rf *" */
+  if (!f1) {
+    strncpy(buf,"pnmtopng > ",12);
+    strncpy(buf+11,nam,111); buf[123]=0;
+    strncpy(buf+strlen(buf),".png",5); 
+    f1 = popen(buf, "w"); if(f1) f1t=1; else E0("popen pnmtopng");
+  }
+  if (!f1) {
+    strncpy(buf,"gzip -c > ",11);
+    strncpy(buf+10,nam,109); buf[120]=0;
+    strncpy(buf+strlen(buf),".ppm.gz",8); 
+    f1 = popen(buf, "w"); if(f1) f1t=1; else E0("popen gzip -c");
+  }
+#endif
+  if(!f1) {
+    strncpy(buf,nam,113); buf[114]=0;
+    strncpy(buf+strlen(buf),".ppm",5);
+    f1=fopen(buf,"wb");
+  }
+  if(!f1) F0("open"); /* open-error */
   fprintf(f1,"P6\n%d %d\n255\n",p->x,p->y);
   if( p->bpp==1 )
   for(y=0;y<p->y;y++)
   for(x=0;x<p->x;x++){
-    rgb[0]=rgb[1]=rgb[2]=p->p[x+y*p->x];
-    if(p->p[x+y*p->x]&4) rgb[0]=64;
-    if(p->p[x+y*p->x]&2) rgb[1]=64;
-    if(p->p[x+y*p->x]&1) rgb[2]=64;
-    if( 1!=(int)fwrite(rgb,3,1,f1) )F0("write");
+    gray=p->p[x+y*p->x];
+    /* replace used bits to get max. contrast, 160=0xA0 */
+    bits=(gray&128)>>4; bits|=bits>>1; bits|=bits>>2;
+//    bits|=bits<<4;
+    rgb[0]=rgb[1]=rgb[2]=(gray & (bits|0xF0)) | bits;
+    if (gray<160) {  /* black */
+      if(gray&2) { rgb[0]|=0x70; rgb[1]&=~0xF0; rgb[2]&=~0xF0; } /* blue */
+      if(gray&4) { rgb[1]|=0x70; rgb[2]&=~0xF0; rgb[0]&=~0xF0; }
+      if(gray&8) { rgb[2]|=0x70; rgb[0]&=~0xF0; rgb[1]&=~0xF0; } /* red */
+      if(gray&1) { rgb[2]|=0x70; rgb[0]|= 0x70; rgb[1]&=~0xF0; } /* cyan */
+    } else {         /* white (box frame) */
+      if(gray&2) { rgb[0]|= 0xF0; rgb[1]&=~0x80; rgb[2]&=~0x80; } /* red */
+      if(gray&4) { rgb[1]|= 0xF0; rgb[2]&=~0x30; rgb[0]&=~0x30; } /* green */
+      if(gray&8) { rgb[2]|= 0xF0; rgb[0]&=~0x20; rgb[1]&=~0x20; } /* blue */
+      if(gray&1) { rgb[2]|= 0xF0; rgb[0]|= 0xF0; rgb[1]&=~0xC0; } /* cyan */
+    }
+    if( 1!=(int)fwrite(rgb,3,1,f1) ) { E0("write"); y=p->y; break; }
   }
   if( p->bpp==3 )
-  if( p->y!=(int)fwrite(p->p,3*p->x,p->y,f1) )F0("write");
-  fclose(f1);
+  if( p->y!=(int)fwrite(p->p,3*p->x,p->y,f1) ) E0("write");
+#ifdef HAVE_POPEN
+  if (f1t) { pclose (f1); f1=NULL; }
+#endif
+  if (f1) fclose(f1);
   return 0;
 }
 
@@ -435,4 +420,3 @@ int writepbm(char *nam,pix *p){// P4 raw-pbm
   return 0;
 }
 // ------------------------------------------------------------------------
-#endif
