@@ -14,22 +14,19 @@
 #include "config.h"
 
 struct pixmap {
-   unsigned char *p;		/* pointer of image buffer (pixmap) */
+   unsigned char *p;	/* pointer of image buffer (pixmap) */
    int x;		/* xsize */
    int y;		/* ysize */
    int bpp;		/* bytes per pixel:  1=gray 3=rgb */
  };
 typedef struct pixmap pix;
 
-#ifdef HAVE_PNM_H
-void readpnm(char *name, pix *p, int vvv);
-#endif
-void readpgm(char *name,pix *p,int vvv);
+void readpgm(char *name, pix *p, int vvv);
 
 /* write pgm-map to pnm-file */
-int writepgm(char *nam,pix *p);
-int writepbm(char *nam,pix *p);
-int writeppm(char *nam,pix *p);		/* use lowest 3 bits for farbcoding */
+int writepgm(char *nam, pix *p);
+int writepbm(char *nam, pix *p);
+int writeppm(char *nam, pix *p); /* use lowest 3 bits for farbcoding */
 
 /* ----- count colors ------ create histogram ------- */
 void makehisto(pix p, unsigned col[256], int vvv);
