@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2006 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -48,42 +48,62 @@ int remove_dust( job_t *job ){
   /* new dust removing  */
   /* FIXME jb:remove pp */
   pix *pp = &job->src.p;
-  int i,j,x,y,ds,x0,x1,y0,y1,df=100000, dd,cs,vvv=JOB->cfg.verbose;
+  int i1,i,j,x,y,x0,x1,y0,y1, cs,vvv=job->cfg.verbose;
   struct box *box2;
-  i=df-1; /* dust size should be revers proportional to its number */
+#define HISTSIZE  100   /* histogramm */
+  int histo[HISTSIZE];
+  cs=job->cfg.cs;
+  /*
+   * count number of black pixels within a box and store it in .dots
+   * later .dots is re-used for number of objects belonging to the character
+   * should be done in the flood-fill algorithm 
+   */
+  for (i1=0;i1<HISTSIZE;i1++) histo[i1]=0;
+  for_each_data(&(job->res.boxlist)) {
+    box2 = (struct box *)list_get_current(&(job->res.boxlist));
+    box2->dots = 0;
+    for(x=box2->x0;x<=box2->x1;x++)
+      for(y=box2->y0;y<=box2->y1;y++){
+        if( pixel(pp,x,y)<cs ){ box2->dots++; }
+      }
+    if (box2->dots<HISTSIZE) histo[box2->dots]++;
+  } end_for_each(&(job->res.boxlist));
+  
   if (job->cfg.dust_size < 0 && job->res.numC > 0) { /* auto detection */
     /* this formula is empirically, high resolution scans have bigger dust */
-    job->cfg.dust_size = ( job->res.sumX/job->res.numC )
-                       * ( job->res.sumY/job->res.numC ) / 100 + 1;
-    if (vvv) fprintf(stderr,"# auto dust size = %d (mX=%d,mY=%d)\n",
-      job->cfg.dust_size,job->res.sumX/job->res.numC,
-                         job->res.sumY/job->res.numC);
-  }
-  dd=job->cfg.dust_size/16+1;	/* step */
-  cs=job->cfg.cs;
-  for( ds=dd;i<df && ds<=job->cfg.dust_size && i>0;ds+=dd )
-  { df=i;i=0;
-    if(vvv){ fprintf(stderr,"# searching dust of size %2d",ds);
-             if(ds>2)fprintf(stderr," and remove detected dust");
-             fprintf(stderr," ..."); 
+    /* maximum allowed dustsize */
+    job->cfg.dust_size = (  ( job->res.sumX/job->res.numC )
+                          * ( job->res.sumY/job->res.numC ) + 64) / 128;
+    /* we assume that for rundum dust applies histo[i+1]<histo[i] */
+    for (i=0;i+1<HISTSIZE && i<job->cfg.dust_size;i++){
+      if (histo[i+1]>=histo[i]/2) break;
     }
-    if(ds==dd){
-      /* count black pixels for each box */
-      for_each_data(&(JOB->res.boxlist)) {
-        box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
-        box2->dots = 0;
-        for(x=box2->x0;x<=box2->x1;x++)
-          for(y=box2->y0;y<=box2->y1;y++){
-            if( pixel(pp,x,y)<cs ){ box2->dots++; }
-          }
-      } end_for_each(&(JOB->res.boxlist));
+    job->cfg.dust_size=i;
+    /* what is the statistic of random dust? 
+     *    if we have p pixels on a x*y image we should have
+     *    (p/(x*y))^1 * (x*y) = p         singlets     
+     *    (p/(x*y))^2 * (x*y) = p^2/(x*y) doublets and
+     *    (p/(x*y))^3 * (x*y) = p^3/(x*y)^2 triplets
+     */
+    if (vvv) fprintf(stderr,"# auto dust size = %d nC= %3d avD= %2d %2d\n",
+                       job->cfg.dust_size, job->res.numC,
+           (job->res.sumX+job->res.numC/2)/job->res.numC,
+           (job->res.sumY+job->res.numC/2)/job->res.numC);
+  }
+  if (job->cfg.dust_size)
+  { i=0;
+    if(vvv){
+       fprintf(stderr,"# remove dust of size %2d",job->cfg.dust_size);
+       fprintf(stderr," histo=%d,%d(?=%d),%d(?=%d)\n# ...",
+          histo[1],histo[2],histo[1]*histo[1]/(pp->x*pp->y),
+          histo[3],histo[1]*histo[1]*histo[1]/(pp->x*pp->y*pp->x*pp->y)); 
     }
     i = 0;
-    for_each_data(&(JOB->res.boxlist)) {
-      box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+    for_each_data(&(job->res.boxlist)) {
+      box2 = (struct box *)list_get_current(&(job->res.boxlist));
       x0=box2->x0;x1=box2->x1;y0=box2->y0;y1=box2->y1;	/* box */
       j=box2->dots;
-      if(j<=ds-dd)      /* remove this tiny object */
+      if(j<=job->cfg.dust_size)      /* remove this tiny object */
       { /* here we should distinguish dust and i-dots,
          * may be we should sort out dots to a seperate dot list and
          * after line detection decide, which is dust and which not
@@ -94,26 +114,26 @@ int remove_dust( job_t *job ){
             continue; /* this idea was to simple, see kscan003.jpg sample */
 #endif
         /* remove from average */
-        JOB->res.numC--;
-        JOB->res.sumX-=x1-x0+1;
-        JOB->res.sumY-=y1-y0+1;
+        job->res.numC--;
+        job->res.sumX-=x1-x0+1;
+        job->res.sumY-=y1-y0+1;
         /* remove pixels (should only be done with dust) */
         for(x=x0;x<=x1;x++)
         for(y=y0;y<=y1;y++){ put(pp,x,y,0,255&~7); }
         /* remove from list */
-	list_del(&(JOB->res.boxlist),box2);
+	list_del(&(job->res.boxlist),box2);
 	/* free memory */
-	free(box2);
+	free_box(box2);
+	i++; /* count as dust particle */
 	continue;
       }
-      if(j<=ds  ) i++; /* count as dust particle */
-    } end_for_each(&(JOB->res.boxlist));
-    if(vvv)fprintf(stderr," %3d cluster detected\n",i);
+    } end_for_each(&(job->res.boxlist));
+    if(vvv)fprintf(stderr," %3d cluster removed, nC= %3d\n",i,job->res.numC);
   }
   /* reset dots to 0 and remove white pixels (new) */
   i=0;
-  for_each_data(&(JOB->res.boxlist)) {
-    box2 = ((struct box *)list_get_current(&(JOB->res.boxlist)));
+  for_each_data(&(job->res.boxlist)) {
+    box2 = ((struct box *)list_get_current(&(job->res.boxlist)));
     box2->dots = 0;
     x0=box2->x0;x1=box2->x1;y0=box2->y0;y1=box2->y1;	/* box */
     if (x1-x0>16 && y1-y0>30) /* only on large enough chars */
@@ -128,64 +148,97 @@ int remove_dust( job_t *job ){
         put(pp,x,y,0,0); i++;  /* (x and 0) or 0 */
       }
     }
-  } end_for_each(&(JOB->res.boxlist));
-  if(vvv)fprintf(stderr,"# %3d white pixels removed, cs=%d\n",i,cs);
+  } end_for_each(&(job->res.boxlist));
+  if (vvv) fprintf(stderr,"# ... %3d white pixels removed, cs=%d nC= %3d\n",
+     i,cs,job->res.numC);
   return 0;
 }
 
 /* ---- smooth big chars ---------------------------------
-   Big chars often do not have smooth borders, which let fail
-   the engine. Here we smooth the borders of big chars (>20x40).
-*/
+ * Big chars often do not have smooth borders, which let fail
+ * the engine. Here we smooth the borders of big chars (>7x16).
+ * Smoothing is important for b/w scans, where we often have
+ * comb like pattern on a vertikal border. I also received
+ * samples with lot of white pixels (sample: 04/02/25).
+ */
 int smooth_borders( job_t *job ){
     pix *pp = &job->src.p;
-    int ii=0,x,y,x0,x1,y0,y1,dx,dy,cs,i0,i1,i2,
-        cn[8],cm,vvv=JOB->cfg.verbose; /* dust found */
+    int ii=0,x,y,x0,x1,y0,y1,dx,dy,cs,i0,i1,i2,i3,i4,n1,n2,
+        cn[8],cm,vvv=job->cfg.verbose; /* dust found */
     struct box *box2;
-    cs=JOB->cfg.cs;
-    if(vvv){ fprintf(stderr,"# smooth big chars 20x39 cs=%d",cs); }
+    cs=job->cfg.cs; n1=n2=0;
+    if(vvv){ fprintf(stderr,"# smooth big chars 7x16 cs=%d",cs); }
     /* filter for each big box */
-    for_each_data(&(JOB->res.boxlist)) {
-        box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+    for_each_data(&(job->res.boxlist)) { n2++; /* count boxes */
+        box2 = (struct box *)list_get_current(&(job->res.boxlist));
         /* do not touch small characters! but how we define small characters? */
         if (box2->x1-box2->x0+1<7 || box2->y1-box2->y0+1<16 ) continue;
         if (box2->c==PICTURE) continue;
-        if (mean_thickness(box2)<5) continue;
+        if (mean_thickness(box2)<3) continue;
+        n1++; /* count boxes matching big-char criteria */
         x0=box2->x0;        y0=box2->y0;
         x1=box2->x1;        y1=box2->y1;
         dx=x1-x0+1;         dy=y1-y0-1;
         /* out_x(box2);
          * dont change to much! only change if absolutely sure!
-         *             .......
-         *       ex:   .?#####
-         *             .......
+         *             .......    1 2 3
+         *       ex:   .?#####    0 * 4
+         *             .......    7 6 5
+         * we should also avoid removing lines by sytematic remove
+         * from left end to the right, so we concern also about distance>1  
          */
         for(x=box2->x0;x<=box2->x1;x++)
-         for(y=box2->y0;y<=box2->y1;y++){ /* filter out high frequenzies */
+         for(y=box2->y0;y<=box2->y1;y++){ /* filter out high frequencies */
            /* this is a very primitive solution, only for learning */
-           cn[0]=pixel(pp,x-1,y);   cn[4]=pixel(pp,x+1,y); // horizontal
-           cn[2]=pixel(pp,x,y-1);   cn[6]=pixel(pp,x,y+1); // vertical
-           cn[1]=pixel(pp,x-1,y-1); cn[3]=pixel(pp,x+1,y-1); // diagonal
-           cn[7]=pixel(pp,x-1,y+1); cn[5]=pixel(pp,x+1,y+1);
+           cn[0]=pixel(pp,x-1,y);
+           cn[4]=pixel(pp,x+1,y);   /* horizontal */
+           cn[2]=pixel(pp,x,y-1);
+           cn[6]=pixel(pp,x,y+1);   /* vertical */
+           cn[1]=pixel(pp,x-1,y-1);
+           cn[3]=pixel(pp,x+1,y-1); /* diagonal */
+           cn[7]=pixel(pp,x-1,y+1);
+           cn[5]=pixel(pp,x+1,y+1);
            cm=pixel(pp,x,y);
-           // check for 5 other and 3 same surrounding pixels
+           /* check for 5 other and 3 same surrounding pixels */
            for (i0=0;i0<8;i0++)
              if ((cn[i0            ]<cs)==(cm<cs)
-              && (cn[(i0+7)     & 7]<cs)!=(cm<cs)) break; // first same
+              && (cn[(i0+7)     & 7]<cs)!=(cm<cs)) break; /* first same */
            for (i1=0;i1<8;i1++)
-             if ((cn[(i0+i1)    & 7]<cs)==(cm<cs)) break; // num same
+             if ((cn[(i0+i1)    & 7]<cs)!=(cm<cs)) break; /* num same */
            for (i2=0;i2<8;i2++)
-             if ((cn[(i0+i1+i2) & 7]<cs)!=(cm<cs)) break; // num other
-           if (i1==3 && i2==5) {  // change!
-             ii++;
-             put(pp,x,y,0,((cm<cs)?cs:cs-8)&~7);
-             // printf(" xy c1 c2 cm new %3d %3d  %3d %3d %3d %3d\n",x-box2->x0,y-box2->y0,c1,c2,cm,pixel(pp,x,y));
+             if ((cn[(i0+i1+i2) & 7]<cs)==(cm<cs)) break; /* num other */
+           cn[0]=pixel(pp,x-2,y);
+           cn[4]=pixel(pp,x+2,y);   /* horizontal */
+           cn[2]=pixel(pp,x,y-2);
+           cn[6]=pixel(pp,x,y+2);   /* vertical */
+           cn[1]=pixel(pp,x-2,y-2);
+           cn[3]=pixel(pp,x+2,y-2); /* diagonal */
+           cn[7]=pixel(pp,x-2,y+2);
+           cn[5]=pixel(pp,x+2,y+2);
+           /* check for 5 other and 3 same surrounding pixels */
+           for (i0=0;i0<8;i0++)
+             if ((cn[i0            ]<cs)==(cm<cs)
+              && (cn[(i0+7)     & 7]<cs)!=(cm<cs)) break; /* first same */
+           for (i3=0;i3<8;i3++)
+             if ((cn[(i0+i3)    & 7]<cs)!=(cm<cs)) break; /* num same */
+           for (i4=0;i4<8;i4++)
+             if ((cn[(i0+i3+i4) & 7]<cs)==(cm<cs)) break; /* num other */
+           if (i1<=3 && i2>=5 && i3>=3 && i4>=3) { /* change only on borders */
+             ii++;             /*   white    : black */
+             put(pp,x,y,7,((cm<cs)?(cs|32):cs/2)&~7);
+#if 0
+             printf(" x y i0 i1 i2 i3 i4 cm new cs %3d %3d"
+             "  %3d %3d %3d %3d %3d  %3d %3d %3d\n",
+              x-box2->x0,y-box2->y0,i0,i1,i2,i3,i3,cm,pixel(pp,x,y),cs);
+#endif
            }
         }
-        // out_x(box2);
-    } end_for_each(&(JOB->res.boxlist));
-    if(vvv)fprintf(stderr," ... %3d changes\n",ii);
-  return 0;
+#if 0  /* debugging */
+        out_x(box2);
+#endif
+    } end_for_each(&(job->res.boxlist));
+    if(vvv)fprintf(stderr," ... %3d changes in %d of %d\n",ii,n1,n2);
+    return 0;
 }
 
 /* test if a corner of box1 is within box2 */
@@ -213,74 +266,86 @@ int box_covered( struct box *box1, struct box *box2){
  */
 int remove_pictures( job_t *job){
   struct box *box4,*box2;
-  int j=0, num_del=0;
+  int j=0, j2=0, num_del=0;
 
-  if (JOB->cfg.verbose)
+  if (job->cfg.verbose)
     fprintf(stderr, "# remove boxes on border");
 
+  /* output a list for picture handle scripts */
+  j=0; j2=0;
+  if(job->cfg.verbose)
+  for_each_data(&(job->res.boxlist)) {
+    box4 = (struct box *)list_get_current(&(job->res.boxlist));
+    if( box4->c==PICTURE ) j++; else j2++;
+  } end_for_each(&(job->res.boxlist));
+  if (job->cfg.verbose)
+    fprintf(stderr," pictures= %d  rest= %d  numC= %d\n# ...",j,j2,job->res.numC);
+
   /* remove dark-border-boxes (typical for copy of book site) */
   for(j=1;j;){ j=0; /* this is only because list_del does not work */
-    for_each_data(&(JOB->res.boxlist)) {
-      box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+    for_each_data(&(job->res.boxlist)) {
+      box2 = (struct box *)list_get_current(&(job->res.boxlist));
       if( box2->c==PICTURE
-#if 0
-       && ( (box2->x0==0            && box2->y0==0)  /* on corner? */
-         || (box2->x0==0            && box2->y1==box2->p->y-1)
-         || (box2->x1==box2->p->x-1 && box2->y1==box2->p->y-1)
-         || (box2->x1==box2->p->x-1 && box2->y0==0)
-#else  /* better if image is parted by lines */
        && ( (box2->x0==0            || box2->y0==0)  /* on border? */
          || (box2->x1==box2->p->x-1 || box2->y1==box2->p->y-1)
          || (box2->x1-box2->x0+1>box2->p->x/2  /* big table? */
           && box2->y1-box2->y0+1>box2->p->y/2)
-#endif
           )
       ){ j=0;
         /* count boxes nested with the picture */
-        for_each_data(&(JOB->res.boxlist)) {
-          box4 = (struct box *)list_get_current(&(JOB->res.boxlist));
+        for_each_data(&(job->res.boxlist)) {
+          box4 = (struct box *)list_get_current(&(job->res.boxlist));
           if( box4 != box2 )  /* not count itself */
           if (box_nested(box4,box2)) j++;  /* box4 in box2 */
-        } end_for_each(&(JOB->res.boxlist));
+        } end_for_each(&(job->res.boxlist));
         if( j>8 ){ /* remove box if more than 8 chars are within box */
-          list_del(&(JOB->res.boxlist), box2); /* does not work proper ?! */
-          if (box2->obj) free(box2->obj);
-          free(box2); j=1; num_del++; break;
+          list_del(&(job->res.boxlist), box2); /* does not work proper ?! */
+          free_box(box2); j=1; num_del++; break;
         } else j=0;
       }
-    } end_for_each(&(JOB->res.boxlist));
+    } end_for_each(&(job->res.boxlist));
   }
 
-  if (JOB->cfg.verbose)
-    fprintf(stderr, " %d, within pictures ", num_del);
+  if (job->cfg.verbose)
+    fprintf(stderr, " deleted= %d, within pictures ", num_del);
   num_del=0;
+
+  /* output a list for picture handle scripts */
+  j=0; j2=0;
+  if(job->cfg.verbose)
+  for_each_data(&(job->res.boxlist)) {
+    box4 = (struct box *)list_get_current(&(job->res.boxlist));
+    if( box4->c==PICTURE ) j++; else j2++;
+  } end_for_each(&(job->res.boxlist));
+  if (job->cfg.verbose)
+    fprintf(stderr," pictures= %d  rest= %d  numC= %d\n# ...",j,j2,job->res.numC);
   
   for(j=1;j;){ j=0;  /* this is only because list_del does not work */
-    if (JOB->cfg.verbose)
-      fprintf(stderr, ".");  // can be slow on gray images
-    for_each_data(&(JOB->res.boxlist)) {
-      box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
-      if( box2->c==PICTURE && !box2->obj)
+    if (job->cfg.verbose)
+      fprintf(stderr, ".");  /* can be slow on gray images */
+    for_each_data(&(job->res.boxlist)) {
+      box2 = (struct box *)list_get_current(&(job->res.boxlist));
+      if( box2->c==PICTURE && box2->num_ac==0)
       for(j=1;j;){ // let it grow to max before leave
         j=0; box4=NULL;
         /* find boxes nested with the picture and remove */
         /* its for pictures build by compounds */
-        for_each_data(&(JOB->res.boxlist)) {
-          box4 = (struct box *)list_get_current(&(JOB->res.boxlist));
+        for_each_data(&(job->res.boxlist)) {
+          box4 = (struct box *)list_get_current(&(job->res.boxlist));
           if(  box4!=box2   /* not destroy self */
-           && (!box4->obj)  /* dont remove barcodes and special objects */ 
-           && (box4->c==UNKNOWN
-            || box4->c==PICTURE) ) /* dont remove valid chars */
+           && (box4->num_ac==0)  /* dont remove barcodes etc. */ 
+           && (/* box4->c==UNKNOWN || */
+                  box4->c==PICTURE) ) /* dont remove valid chars */
           if(
              /* box4 in box2, +1..-1 frame for pixel-patterns */
                box_nested(box4,box2)
              /* or box2 in box4 */
             || box_nested(box2,box4) /* same? */
               )
-          if (  box4->x1-box4->x0+1>2*JOB->res.avX
-             || box4->x1-box4->x0+1<JOB->res.avX/2
-             || box4->y1-box4->y0+1>2*JOB->res.avY
-             || box4->y1-box4->y0+1<JOB->res.avY/2
+          if (  box4->x1-box4->x0+1>2*job->res.avX
+             || box4->x1-box4->x0+1<job->res.avX/2
+             || box4->y1-box4->y0+1>2*job->res.avY
+             || box4->y1-box4->y0+1<job->res.avY/2
              || box_covered(box4,box2) )   /* box4 completely within box2 */ 
             /* dont remove chars! see rotate45.fig */
           {
@@ -293,33 +358,33 @@ int remove_pictures( job_t *job){
             j=1;   // mark box4 as valid
             break; // and leave inner loop
           }
-        } end_for_each(&(JOB->res.boxlist));
+        } end_for_each(&(job->res.boxlist));
         if (j!=0 && box4!=NULL) { // check for valid box4
-          list_del(&(JOB->res.boxlist), box4); /* does not work proper ?! */
-          free(box4); // break;  // ToDo: necessary to leave after del???
+          list_del(&(job->res.boxlist), box4); /* does not work proper ?! */
+          free_box(box4); // break;  // ToDo: necessary to leave after del???
           num_del++;
         }
         
       }
-    } end_for_each(&(JOB->res.boxlist)); 
+    } end_for_each(&(job->res.boxlist)); 
   }
 
-  if (JOB->cfg.verbose)
-    fprintf(stderr, " %d,", num_del);
+  if (job->cfg.verbose)
+    fprintf(stderr, " deleted= %d,", num_del);
 
   /* output a list for picture handle scripts */
-  j=0;
-  if(JOB->cfg.verbose)
-  for_each_data(&(JOB->res.boxlist)) {
-    box4 = (struct box *)list_get_current(&(JOB->res.boxlist));
+  j=0; j2=0;
+  if(job->cfg.verbose)
+  for_each_data(&(job->res.boxlist)) {
+    box4 = (struct box *)list_get_current(&(job->res.boxlist));
     if( box4->c==PICTURE ) {
       fprintf(stderr,"\n# ... found picture at %4d %4d size %4d %4d",
          box4->x0, box4->y0, box4->x1-box4->x0+1, box4->y1-box4->y0+1 );
       j++;
-    }
-  } end_for_each(&(JOB->res.boxlist));
-  if (JOB->cfg.verbose)
-    fprintf(stderr," %d cluster detected, boxes %d\n",j,JOB->res.numC);
+    } else j2++;
+  } end_for_each(&(job->res.boxlist));
+  if (job->cfg.verbose)
+    fprintf(stderr," pictures= %d  rest= %d  numC= %d\n",j,j2,job->res.numC);
   return 0;
 }
 
@@ -342,6 +407,7 @@ int remove_melted_serifs( pix *pp ){
   if(vvv){ fprintf(stderr,"# searching melted serifs ..."); }
   for_each_data(&(JOB->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+    if (box2->c != UNKNOWN) continue; /* dont try on pictures */
     x0=box2->x0; x1=box2->x1; 
     y0=box2->y0; y1=box2->y1;	// box
     // upper serifs
@@ -378,7 +444,8 @@ int remove_melted_serifs( pix *pp ){
 	if(vvv&4){ 
 	  fprintf(stderr,"\n"); 
 	  out_x(box2);
-	  fprintf(stderr,"# melted serifs corrected on %d %d j1=%d j3=%d",j2-x0,y-y0,j1-x0,j3-x0);
+	  fprintf(stderr,"# melted serifs corrected on %d %d j1=%d j3=%d",
+	    j2-x0, y, j1-x0, j3-x0);
 	}
 	for(xb=0,xa=0;xa<(x1-x0+4)/8;xa++){ /* detect vertical gap */
 	  i3=y1; 
@@ -394,8 +461,9 @@ int remove_melted_serifs( pix *pp ){
 	  box3=malloc_box(box2);
 	  box3->x1=j2-1;
 	  box2->x0=j2+1; x1=box2->x1;
+	  cut_box(box2); /* cut vectors outside the box */
+	  cut_box(box3);
 	  box3->num=JOB->res.numC;
-	  box3->obj=NULL;
 	  list_ins(&(JOB->res.boxlist),box2,box3); JOB->res.numC++; ii++; /* insert box3 before box2 */
 	  if(vvv&4) fprintf(stderr," => splitted");
 	  j1=x0=box2->x0; x=0; /* hopefully ok, UVW */
@@ -457,8 +525,9 @@ int remove_melted_serifs( pix *pp ){
 	  box3=malloc_box(box2);
 	  box3->x1=j2-1;
 	  box2->x0=j2; x1=box2->x1;
+	  cut_box(box2); /* cut vectors outside the box */
+	  cut_box(box3);
 	  box3->num=JOB->res.numC;
-	  box3->obj=0;
 	  list_ins(&(JOB->res.boxlist),box2,box3); JOB->res.numC++; ii++;
 	  /* box3,box2 in correct order??? */
 	  if(vvv&4) fprintf(stderr," => splitted");
@@ -497,7 +566,7 @@ int remove_rest_of_dust() {
 	 JOB->res.numC--;
 	 i++;
 	 list_del(&(JOB->res.boxlist), box2);
-	 free(box2);
+	 free_box(box2);
       }
     }
   } end_for_each(&(JOB->res.boxlist));
@@ -518,7 +587,7 @@ int remove_rest_of_dust() {
 	      JOB->res.numC--;
 	      j++;
 	      list_del(&(JOB->res.boxlist), box4);
-	      free(box4);
+	      free_box(box4);
 	    }
 	  }
 	} end_for_each(&(JOB->res.boxlist));
