@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000  Joerg Schulenburg
+Copyright (C) 2000-2006  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -271,8 +271,11 @@ wchar_t compose(wchar_t main, wchar_t modifier) {
 	case 'E':	    return LATIN_CAPITAL_LETTER_E_WITH_DOT_ABOVE;
 	case 'g':	    return LATIN_SMALL_LETTER_G_WITH_DOT_ABOVE;
 	case 'G':	    return LATIN_CAPITAL_LETTER_G_WITH_DOT_ABOVE;
+	case 'l':	    return 'i';  /* correct wrong recognition */
+	case 'i':	    return 'i';
 	case LATIN_SMALL_LETTER_DOTLESS_I:	    return 'i';
 	case 'I':	    return LATIN_CAPITAL_LETTER_I_WITH_DOT_ABOVE;
+	case 'j':	    return 'j';
 	case 'o':	    return LATIN_SMALL_LETTER_O_WITH_DOT_ABOVE;
 	case 'O':	    return LATIN_CAPITAL_LETTER_O_WITH_DOT_ABOVE;
 	case 'z':	    return LATIN_SMALL_LETTER_Z_WITH_DOT_ABOVE;
@@ -482,10 +485,10 @@ const char *decode(wchar_t c, FORMAT type) {
 	case UNKNOWN:
 	  return (const char *)"_";
 	case PICTURE:
-	  return (const char *)"(PICTURE)";
+	  return (const char *)"_"; /* Due to Mobile OCR */
 		
 	default:
-	  /* snprintf seems to be no standart, so I use insecure sprintf */
+	  /* snprintf seems to be no standard, so I use insecure sprintf */
 	  sprintf(buf,"\\code(%04x)",(unsigned)c);
 	  return buf;  // UNDEFINED;
       }
@@ -525,17 +528,17 @@ const char *decode(wchar_t c, FORMAT type) {
 	case INVERTED_EXCLAMATION_MARK:
 	  return (const char *)"!'";
 	case CENT_SIGN:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\textcent"; /* \usepackage{textcomp} */
 	case POUND_SIGN:
 	  return (const char *)"\\pounds";
 	case EURO_CURRENCY_SIGN:
 	  return (const char *)"\\euro"; /* \usepackage{eurosans} */
 	case CURRENCY_SIGN:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\textcurrency"; /* \usepackage{textcomp} */
 	case YEN_SIGN:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\textyen"; /* \usepackage{textcomp} */
 	case BROKEN_BAR:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\textbrokenbar"; /* \usepackage{textcomp} */
 	case SECTION_SIGN:
 	  return (const char *)"\\S";
 	case DIAERESIS:
@@ -551,9 +554,9 @@ const char *decode(wchar_t c, FORMAT type) {
 	case SOFT_HYPHEN:
 	  return (const char *)"\\-";
 	case REGISTERED_SIGN:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\textregistered";/* \usepackage{textcomp} */
 	case MACRON:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\textasciimacron";/* \usepackage{textcomp} */
 	case DEGREE_SIGN:
 	  return (const char *)"$^{o}$";
 	case PLUS_MINUS_SIGN:
@@ -776,7 +779,7 @@ const char *decode(wchar_t c, FORMAT type) {
 	case GREEK_CAPITAL_LETTER_PHI:
 	  return (const char *)"\\( \\Phi \\)";
 	case GREEK_CAPITAL_LETTER_CHI:
-	  return (const char *)UNDEFINED;
+	  return (const char *)"\\( \\Chi \\)";
 	case GREEK_CAPITAL_LETTER_PSI:
 	  return (const char *)"\\( \\Psi \\)";
 	case GREEK_CAPITAL_LETTER_OMEGA:
@@ -939,16 +942,20 @@ const char *decode(wchar_t c, FORMAT type) {
 	case PICTURE:
 	  return (const char *)"(PICTURE)";
 	default:
-	  /* snprintf seems to be no standart, so I use insecure sprintf */
+	  /* snprintf seems to be no standard, so I use insecure sprintf */
 	  sprintf(buf,"\\symbol{%u}",(unsigned)c);
 	  return buf;  // UNDEFINED;
 	}
     case HTML:
-    case XML:
       if ( c >= SPACE && c <= TILDE ) { /* ASCII */
         switch (c) {
           case '&':
             return (const char *)"&amp;";
+          /* semicolon must not be coded */
+          case '\'':
+            return (const char *)"&apos;";
+          case '"':
+            return (const char *)"&quot;";
           case '<':
             return (const char *)"&lt;";
           case '>':
@@ -1190,6 +1197,41 @@ const char *decode(wchar_t c, FORMAT type) {
 	  return buf;  // undefined
       }
       break;
+    case XML:   /* only 5 &xxx;-ENTITIES ar defined by default */
+      if ( c >= SPACE && c <= TILDE ) { /* ASCII */
+        switch (c) {
+          case '&':
+            return (const char *)"&amp;";
+          case '\'':
+            return (const char *)"&apos;";
+          case '"':
+            return (const char *)"&quot;";
+          case '<':
+            return (const char *)"&lt;";
+          case '>':
+            return (const char *)"&gt;";
+        }
+        buf[0] = (char)c;
+        return buf;
+      }
+      switch (c) {    /* subject of change! */
+	case PICTURE:
+	  return (const char *)"(PICTURE)";
+        case UNKNOWN:
+          return (const char *)"_"; // better use colored symbol?
+        case LINE_FEED:             // \n handled somwhere else?
+	case FORM_FEED:
+	case CARRIAGE_RETURN:
+	  return (const char *)"<br />";
+	case NO_BREAK_SPACE:
+	  return (const char *)"<nobr />";
+	case 0:
+	  return (const char *)"";
+	default:
+	  sprintf(buf,"&#x%03x;",(unsigned)c);
+	  return buf;  // undefined
+      }
+      break;
     case SGML:
       switch (c) {
 	default:
@@ -1207,10 +1249,10 @@ const char *decode(wchar_t c, FORMAT type) {
 	case UNKNOWN:
 	  return (const char *)"(?)";
 	case PICTURE:
-	  return (const char *)"(PICTURE)";
+	  return (const char *)"(?)";
 		
 	default:
-	  /* snprintf seems to be no standart, so I use insecure sprintf */
+	  /* snprintf seems to be no standard, so I use insecure sprintf */
 	  if ((unsigned)c>255) sprintf(buf,"(0x%04x)",(unsigned)c);
 	  else                 sprintf(buf,"(0x%02x)",(unsigned)c);
 	  return buf;  // UNDEFINED;
@@ -1219,24 +1261,51 @@ const char *decode(wchar_t c, FORMAT type) {
     default: /* use UTF8 as default, test with xterm -u8 */
       /* extra */
       if ( c == UNKNOWN )  return (const char *)"_";
-      if ( c == PICTURE )  return (const char *)"(PICTURE)";
-      if ( c <= 0x7F ) { /* UTF8 == 7bit ASCII */
+      if ( c == PICTURE )  return (const char *)"_"; /* Due to Mobile OCR */
+      if ( c <= 0x0000007F ) {      /* UTF8 == 7bit ASCII */
         buf[0] = (char)c;
         return buf;
       }
-      if ( c <= (wchar_t)0x7FF ) { /* UTF8 == 11bit */
-        buf[0] = (char)(0xc0+(c>>6 & 0x1f)); /* 110xxxxx */
-        buf[1] = (char)(0x80+(c    & 0x3f)); /* 10xxxxxx */
+      if ( c <= (wchar_t)0x000007FF ) {  /* UTF8 == 11bit */
+        buf[0] = (char)(0xc0|(c>> 6 & 0x1f)); /* 110xxxxx */
+        buf[1] = (char)(0x80|(c     & 0x3f)); /* 10xxxxxx */
         buf[2] = (char)0; /* terminate string */
         return buf;
       }
-      if ( c <= (wchar_t)0xFFFF ) { /* UTF8 == 16bit */
-        buf[0] = (char)(0xd0+(c>>12 & 0x0f)); /* 1110xxxx */
-        buf[1] = (char)(0x80+(c>> 6 & 0x3f)); /* 10xxxxxx */
-        buf[2] = (char)(0x80+(c     & 0x3f)); /* 10xxxxxx */
+      if ( c <= (wchar_t)0x0000FFFF ) {  /* UTF8 == 16bit */
+        buf[0] = (char)(0xe0|(c>>12 & 0x0f)); /* 1110xxxx */
+        buf[1] = (char)(0x80|(c>> 6 & 0x3f)); /* 10xxxxxx */
+        buf[2] = (char)(0x80|(c     & 0x3f)); /* 10xxxxxx */
         buf[3] = (char)0; /* terminate string */
         return buf;
       }
+      if ( c <= (wchar_t)0x001FFFFF ) {  /* UTF8 == 21bit */
+        buf[0] = (char)(0xf0|(c>>18 & 0x07)); /* 11110xxx */
+        buf[1] = (char)(0x80|(c>>12 & 0x3f)); /* 10xxxxxx */
+        buf[2] = (char)(0x80|(c>> 6 & 0x3f)); /* 10xxxxxx */
+        buf[3] = (char)(0x80|(c     & 0x3f)); /* 10xxxxxx */
+        buf[4] = (char)0; /* terminate string */
+        return buf;
+      }
+      if ( c <= (wchar_t)0x03FFFFFF ) {  /* UTF8 == 26bit */
+        buf[0] = (char)(0xf8|(c>>24 & 0x03)); /* 111110xx */
+        buf[1] = (char)(0x80|(c>>18 & 0x3f)); /* 10xxxxxx */
+        buf[2] = (char)(0x80|(c>>12 & 0x3f)); /* 10xxxxxx */
+        buf[3] = (char)(0x80|(c>> 6 & 0x3f)); /* 10xxxxxx */
+        buf[4] = (char)(0x80|(c     & 0x3f)); /* 10xxxxxx */
+        buf[5] = (char)0; /* terminate string */
+        return buf;
+      }
+      if ( c <= (wchar_t)0x7FFFFFFF ) {  /* UTF8 == 31bit */
+        buf[0] = (char)(0xfc|(c>>30 & 0x01)); /* 1111110x */
+        buf[1] = (char)(0x80|(c>>24 & 0x3f)); /* 10xxxxxx */
+        buf[2] = (char)(0x80|(c>>18 & 0x3f)); /* 10xxxxxx */
+        buf[3] = (char)(0x80|(c>>12 & 0x3f)); /* 10xxxxxx */
+        buf[4] = (char)(0x80|(c>> 6 & 0x3f)); /* 10xxxxxx */
+        buf[5] = (char)(0x80|(c     & 0x3f)); /* 10xxxxxx */
+        buf[6] = (char)0; /* terminate string */
+        return buf;
+      }
       return (const char *)UNDEFINED;
   }
 }
